package edu.uoc.tfg.nutriknow.adapters;

import android.content.Context;
import android.content.res.ColorStateList;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseExpandableListAdapter;
import android.widget.ExpandableListView;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.List;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.Additives;
import edu.uoc.tfg.nutriknow.model.AdditivesClasses;

/**
 * Adapter for managing the expandable list of additives in views
 */
public class AdditivesAdapter extends BaseExpandableListAdapter {

    private static final String ES = "es";
    private static final String EN = "en";

    private final List<Additives> additives;
    private final String language;
    private final Context context;
    private final ExpandableListView listView;

    /**
     * Adapter constructor
     *
     * @param additives additives list
     * @param ctx       context
     * @param language  selected locale
     * @param listView  view to fill
     */
    public AdditivesAdapter(List<Additives> additives, Context ctx, String language, ExpandableListView listView) {
        this.additives = additives;
        this.context = ctx;
        this.language = language;
        this.listView = listView;
    }

    /**
     * @return size of the top level list
     */
    @Override
    public int getGroupCount() {
        return additives.size();
    }

    /**
     * @param groupPosition position in the list
     * @return size of the bottom level list
     */
    @Override
    public int getChildrenCount(int groupPosition) {
        return ((Additives) getGroup(groupPosition)).additives_classes.size();
    }

    /**
     * Get the content of the top level list in the given position
     *
     * @param groupPosition position in the list
     * @return content
     */
    @Override
    public Object getGroup(int groupPosition) {
        return additives.get(groupPosition);
    }

    /**
     * Get the content of the boottom level list in the given position
     *
     * @param groupPosition position in the top level list
     * @param childPosition position in the bottom level list
     * @return content
     */
    @Override
    public Object getChild(int groupPosition, int childPosition) {
        return ((Additives) getGroup(groupPosition)).additives_classes.get(childPosition);
    }

    /**
     * Get the position in the top level list
     *
     * @param groupPosition position
     * @return position
     */
    @Override
    public long getGroupId(int groupPosition) {
        return groupPosition;
    }

    /**
     * Get the position in the bottom level list
     *
     * @param groupPosition position
     * @return position
     */
    @Override
    public long getChildId(int groupPosition, int childPosition) {
        return childPosition;
    }

    /**
     * @return boolean
     */
    @Override
    public boolean hasStableIds() {
        return false;
    }

    /**
     * Create the view for the top level list
     *
     * @param groupPosition position of the list
     * @param isExpanded    boolean
     * @param convertView   view to inflate
     * @param parent        parent view
     * @return view
     */
    @Override
    public View getGroupView(int groupPosition, boolean isExpanded, View convertView, ViewGroup parent) {
        // Define source list
        Additives additive = (Additives) getGroup(groupPosition);

        // create main items (groups)
        if (convertView == null) {
            LayoutInflater layoutInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            convertView = layoutInflater.inflate(R.layout.product_info_row_layout, null);
        }

        // Reference the different views in the layout
        ImageView identificativeImageView = (ImageView) convertView.findViewById(R.id.identificative_icon);
        TextView labelTextView = (TextView) convertView.findViewById(R.id.label);

        identificativeImageView.setImageDrawable(context.getDrawable(R.drawable.ic_additive));
        identificativeImageView.setImageTintList(ColorStateList.valueOf(context.getColor(R.color.backgroundColorPrimary)));
        if (ES.equals(language)) {
            labelTextView.setText(additive.name_es);
        } else {
            labelTextView.setText(additive.name_en);
        }
        // The list should be always expanded
        listView.expandGroup(groupPosition);
        convertView.setClickable(false);

        return convertView;
    }

    /**
     * Create the view for the bottom level list
     *
     * @param groupPosition position of the top list
     * @param childPosition position of the bottom list
     * @param isLastChild   boolean
     * @param convertView   view to inflate
     * @param parent        parent view
     * @return view
     */
    @Override
    public View getChildView(int groupPosition, int childPosition, boolean isLastChild, View convertView, ViewGroup parent) {
        // Define source list
        AdditivesClasses.AdditiveClass additiveClass = (AdditivesClasses.AdditiveClass) getChild(groupPosition, childPosition);

        // create bottom items (children)
        if (convertView == null) {
            LayoutInflater layoutInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            convertView = layoutInflater.inflate(R.layout.product_info_child_row_layout, null);
        }

        // Reference the different views in the layout
        ImageView identificativeImageView = (ImageView) convertView.findViewById(R.id.identificative_icon);
        TextView labelTextView = (TextView) convertView.findViewById(R.id.label);

        identificativeImageView.setImageDrawable(context.getDrawable(R.drawable.ic_light_bulb));
        identificativeImageView.setImageTintList(ColorStateList.valueOf(context.getColor(R.color.backgroundColorPrimary)));
        if (ES.equals(language)) {
            labelTextView.setText(additiveClass.name.es);
        } else {
            labelTextView.setText(additiveClass.name.en);
        }

        return convertView;
    }

    /**
     * @return boolean
     */
    @Override
    public boolean isChildSelectable(int groupPosition, int childPosition) {
        return false;
    }

}
