package edu.uoc.tfg.nutriknow.adapters;

import android.content.Context;
import android.content.res.ColorStateList;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.List;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.Allergens;

/**
 * Adapter for managing the list of allergens and traces in views
 */
public class AllergensSummaryAdapter extends ArrayAdapter<Allergens.Allergen> {

    private final List<Allergens.Allergen> allergensInProduct;
    private List<Allergens.Allergen> allergensInProfile;
    private final Context context;

    /**
     * ViewHolder structure
     */
    private static class ViewHolder {
        ImageView identificativeImageView;
        TextView labelTextView;
        ImageView valueImageView;
    }

    /**
     * Adapter constructor
     *
     * @param allergensInProduct list of allergens found in product
     * @param allergensInProfile list of allergens defined by the user
     * @param ctx                context
     */
    public AllergensSummaryAdapter(List<Allergens.Allergen> allergensInProduct,
                                   List<Allergens.Allergen> allergensInProfile, Context ctx) {
        super(ctx, R.layout.product_info_allergens_row_with_value_layout, allergensInProfile);
        this.allergensInProduct = allergensInProduct;
        this.allergensInProfile = allergensInProfile;
        this.context = ctx;
    }

    /**
     * Adapter constructor
     *
     * @param allergensInProduct list of allergens found in product
     * @param ctx                context
     */
    public AllergensSummaryAdapter(List<Allergens.Allergen> allergensInProduct, Context ctx) {
        super(ctx, R.layout.product_info_row_layout, allergensInProduct);
        this.allergensInProduct = allergensInProduct;
        this.context = ctx;
    }

    /**
     * Create the view
     *
     * @param position    position in the source list
     * @param convertView view to inflate
     * @param parent      parent view
     * @return view
     */
    @Override
    public View getView(int position, View convertView, ViewGroup parent) {

        // Define source list
        Allergens.Allergen allergen = getItem(position);

        // Check if an existing view is being reused, otherwise inflate the view
        ViewHolder viewHolder; // view lookup cache stored in tag
        if (convertView == null) {

            viewHolder = new ViewHolder();
            LayoutInflater inflater = LayoutInflater.from(getContext());
            // Depending which constructor was called, the layout used is different
            if (allergensInProfile == null) {
                // Ingredients tab list
                convertView = inflater.inflate(R.layout.product_info_row_layout, parent, false);
            } else {
                // Summary tab list
                convertView = inflater.inflate(R.layout.product_info_allergens_row_with_value_layout, parent, false);
                viewHolder.valueImageView = (ImageView) convertView.findViewById(R.id.value_icon);
            }
            // Reference the views in the layout
            viewHolder.identificativeImageView = (ImageView) convertView.findViewById(R.id.identificative_icon);
            viewHolder.labelTextView = (TextView) convertView.findViewById(R.id.label);

            convertView.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) convertView.getTag();
        }

        // Define the content of the layout views
        viewHolder.identificativeImageView.setImageDrawable(context.getDrawable(getAllergenDrawable(allergen)));
        viewHolder.identificativeImageView.setImageTintList(ColorStateList.valueOf(context.getColor(R.color.backgroundColorPrimary)));
        viewHolder.labelTextView.setText(allergen.name);
        if (allergensInProfile != null) {
            viewHolder.valueImageView.setImageDrawable(context.getDrawable(getValueDrawable(allergen)));
        }
        return convertView;
    }

    /**
     * Get the drawable for the allergen presence
     *
     * @param allergen allergen to find
     * @return id of the drawable
     */
    private int getValueDrawable(Allergens.Allergen allergen) {
        int drawableId = 0;
        boolean found = false;
        for (Allergens.Allergen allergenInProduct : allergensInProduct) {
            if (allergen.id.equals(allergenInProduct.id)) {
                found = true;
                break;
            }
        }
        if (found) {
            drawableId = R.drawable.high;
        } else {
            drawableId = R.drawable.low;
        }
        return drawableId;
    }

    /**
     * Get the drawable for the allergen representation
     *
     * @param allergen allergen to find
     * @return id of the drawable
     */
    private int getAllergenDrawable(Allergens.Allergen allergen) {
        int drawableId = 0;
        switch (allergen.id) {
            case "en:gluten":
                drawableId = R.drawable.ic_gluten;
                break;
            case "en:crustaceans":
                drawableId = R.drawable.ic_crustaceans;
                break;
            case "en:lupin":
                drawableId = R.drawable.ic_lupin;
                break;
            case "en:soybeans":
                drawableId = R.drawable.ic_soy;
                break;
            case "en:milk":
                drawableId = R.drawable.ic_milk;
                break;
            case "en:mustard":
                drawableId = R.drawable.ic_mustard;
                break;
            case "en:molluscs":
                drawableId = R.drawable.ic_molluscs;
                break;
            case "en:sulphur-dioxide-and-sulphites":
                drawableId = R.drawable.ic_sulphites;
                break;
            case "en:nuts":
                drawableId = R.drawable.ic_nuts;
                break;
            case "en:peanuts":
                drawableId = R.drawable.ic_peanuts;
                break;
            case "en:eggs":
                drawableId = R.drawable.ic_eggs;
                break;
            case "en:sesame-seeds":
                drawableId = R.drawable.ic_sesame;
                break;
            case "en:celery":
                drawableId = R.drawable.ic_celery;
                break;
            case "en:fish":
                drawableId = R.drawable.ic_fish;
                break;
        }
        return drawableId;
    }
}
