package edu.uoc.tfg.nutriknow.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import java.util.List;
import java.util.Locale;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.Allergens;
import edu.uoc.tfg.nutriknow.model.Favorite;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.model.Search;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;
import edu.uoc.tfg.nutriknow.views.fragments.FavoritesFragment;

/**
 * Adapter for managing the list of favorite products
 */
public class FavoritesRecyclerAdapter extends RecyclerView.Adapter<FavoritesRecyclerAdapter.ViewHolder> {

    private static final String ES = "es";

    private final List<Product> products;
    private final List<Favorite> favorites;
    private final WelcomeActivity welcomeActivity;
    private final FavoritesFragment favoritesFragment;

    /**
     * Adapter constructor
     *
     * @param favorites         source list
     * @param products          products list to get allergens content
     * @param welcomeActivity   parent activity
     * @param favoritesFragment fragment where the adapter is used
     */
    public FavoritesRecyclerAdapter(List<Favorite> favorites, List<Product> products,
                                    WelcomeActivity welcomeActivity, FavoritesFragment favoritesFragment) {
        this.welcomeActivity = welcomeActivity;
        this.favoritesFragment = favoritesFragment;
        this.favorites = favorites;
        this.products = products;
    }

    /**
     * ViewHolder structure
     */
    class ViewHolder extends RecyclerView.ViewHolder {

        public TextView itemTitle;
        public TextView itemDetail;
        public ImageView itemCelery;
        public ImageView itemCrustaceans;
        public ImageView itemEggs;
        public ImageView itemFish;
        public ImageView itemGluten;
        public ImageView itemLupin;
        public ImageView itemMilk;
        public ImageView itemMolluscs;
        public ImageView itemMustard;
        public ImageView itemNuts;
        public ImageView itemPeanuts;
        public ImageView itemSesame;
        public ImageView itemSoy;
        public ImageView itemSulphites;

        public ImageView deleteButton;
        public ImageView addFavoriteButton;
        public ImageView newIcon;

        private int selectedIndex;
        private Search selectedSearch;
        private boolean isOtherAction = false;

        public ViewHolder(View itemView) {
            super(itemView);
            // Reference the views in the layout
            itemTitle = (TextView) itemView.findViewById(R.id.item_title);
            itemDetail = (TextView) itemView.findViewById(R.id.item_detail);

            // Icons for pointing out the presence of allergens
            itemCelery = (ImageView) itemView.findViewById(R.id.ic_celery);
            itemCrustaceans = (ImageView) itemView.findViewById(R.id.ic_crustaceans);
            itemEggs = (ImageView) itemView.findViewById(R.id.ic_eggs);
            itemFish = (ImageView) itemView.findViewById(R.id.ic_fish);
            itemGluten = (ImageView) itemView.findViewById(R.id.ic_gluten);
            itemLupin = (ImageView) itemView.findViewById(R.id.ic_lupin);
            itemMilk = (ImageView) itemView.findViewById(R.id.ic_milk);
            itemMolluscs = (ImageView) itemView.findViewById(R.id.ic_molluscs);
            itemMustard = (ImageView) itemView.findViewById(R.id.ic_mustard);
            itemNuts = (ImageView) itemView.findViewById(R.id.ic_nuts);
            itemPeanuts = (ImageView) itemView.findViewById(R.id.ic_peanuts);
            itemSesame = (ImageView) itemView.findViewById(R.id.ic_sesame);
            itemSoy = (ImageView) itemView.findViewById(R.id.ic_soy);
            itemSulphites = (ImageView) itemView.findViewById(R.id.ic_sulphites);

            // Define the behaviour of the delete icon
            deleteButton = (ImageView) itemView.findViewById(R.id.ic_delete);
            deleteButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    isOtherAction = true;
                    itemView.callOnClick();
                    deleteFavorite();
                }
            });
            // Define the behaviour of the add icon
            addFavoriteButton = (ImageView) itemView.findViewById(R.id.ic_add_favorite);
            addFavoriteButton.setVisibility(View.GONE);
            // Define the behaviour of tapping the card in the layout
            itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    selectedIndex = getAdapterPosition();
                    selectedSearch = favorites.get(selectedIndex);
                    if (!isOtherAction) {
                        // Always retrieve product information from rest api
                        favoritesFragment.retrieveProductInformation(favorites.get(selectedIndex).getProductCode());
                    }
                }
            });
            // Icons for pointing out a new product
            newIcon = (ImageView) itemView.findViewById(R.id.ic_is_new);
        }

        /**
         * Delete favorite product from the list
         */
        private void deleteFavorite() {
            isOtherAction = false;
            favoritesFragment.deleteProductFromFavorites(favorites.get(selectedIndex).getProductCode());
        }

    }

    /**
     * Create the view
     *
     * @param viewGroup card layout to inflate
     * @param viewType  type of view
     * @return viewHolder with the inflated card
     */
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup,
                                         int viewType) {
        View v = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.card_layout, viewGroup, false);
        ViewHolder viewHolder = new ViewHolder(v);
        return viewHolder;
    }

    /**
     * Define the content of the viewHolder
     *
     * @param viewHolder   structure to fill
     * @param listPosition position on the source list
     */
    @Override
    public void onBindViewHolder(final ViewHolder viewHolder, final int listPosition) {
        // Depending on the locale, inform the name of the product
        Locale currentLocale = welcomeActivity.getResources().getConfiguration().locale;
        if (ES.equals(currentLocale.getLanguage())) {
            if (favorites.get(listPosition).getProductName_es() != null && !favorites.get(listPosition).getProductName_es().isEmpty()) {
                viewHolder.itemTitle.setText(favorites.get(listPosition).getProductName_es());
            } else {
                if (favorites.get(listPosition).getProductName_en() != null && !favorites.get(listPosition).getProductName_en().isEmpty()) {
                    viewHolder.itemTitle.setText(favorites.get(listPosition).getProductName_en());
                } else {
                    viewHolder.itemTitle.setText(favoritesFragment.getContext().getString(R.string.empty_product_name));
                }

            }
        } else {
            if (favorites.get(listPosition).getProductName_en() != null && !favorites.get(listPosition).getProductName_en().isEmpty()) {
                viewHolder.itemTitle.setText(favorites.get(listPosition).getProductName_en());
            } else {
                if (favorites.get(listPosition).getProductName_es() != null && !favorites.get(listPosition).getProductName_es().isEmpty()) {
                    viewHolder.itemTitle.setText(favorites.get(listPosition).getProductName_es());
                } else {
                    viewHolder.itemTitle.setText(favoritesFragment.getContext().getString(R.string.empty_product_name));
                }

            }
        }
        // Set the barcode content
        viewHolder.itemDetail.setText(favorites.get(listPosition).getProductCode());

        // Retrieve the product related with the favorite item selected
        Product selectedProduct = null;
        for (Product product : products) {
            if (favorites.get(listPosition).getProductCode().equals(product.getSummaryData().getCode())) {
                selectedProduct = product;
                break;
            }
        }
        // Display only the icons for the allergens found in the product
        hideAllIcons(viewHolder);
        displayAllergenIcons(viewHolder, selectedProduct.getIngredientsData().getAllergens_es());
        // Toggle the visibility of the new flag
        if (favorites.get(listPosition).isNew()) {
            viewHolder.newIcon.setVisibility(View.VISIBLE);
        } else {
            viewHolder.newIcon.setVisibility(View.GONE);
        }
    }

    /**
     * Get the total size
     *
     * @return size
     */
    @Override
    public int getItemCount() {
        return favorites.size();
    }

    /**
     * Hide all the icons in the viewHolder
     *
     * @param viewHolder structure where hide the icons
     */
    private void hideAllIcons(ViewHolder viewHolder) {
        viewHolder.itemGluten.setVisibility(View.GONE);
        viewHolder.itemCrustaceans.setVisibility(View.GONE);
        viewHolder.itemLupin.setVisibility(View.GONE);
        viewHolder.itemSoy.setVisibility(View.GONE);
        viewHolder.itemMilk.setVisibility(View.GONE);
        viewHolder.itemMustard.setVisibility(View.GONE);
        viewHolder.itemMolluscs.setVisibility(View.GONE);
        viewHolder.itemSulphites.setVisibility(View.GONE);
        viewHolder.itemNuts.setVisibility(View.GONE);
        viewHolder.itemPeanuts.setVisibility(View.GONE);
        viewHolder.itemEggs.setVisibility(View.GONE);
        viewHolder.itemSesame.setVisibility(View.GONE);
        viewHolder.itemCelery.setVisibility(View.GONE);
        viewHolder.itemFish.setVisibility(View.GONE);
    }

    /**
     * Display the icons of the allergens contained in the product
     *
     * @param viewHolder structure where display the icons
     * @param allergens  list of allergens found
     */
    private void displayAllergenIcons(ViewHolder viewHolder, List<Allergens.Allergen> allergens) {
        for (Allergens.Allergen allergen : allergens) {
            switch (allergen.id) {
                case "en:gluten":
                    viewHolder.itemGluten.setVisibility(View.VISIBLE);
                    break;
                case "en:crustaceans":
                    viewHolder.itemCrustaceans.setVisibility(View.VISIBLE);
                    break;
                case "en:lupin":
                    viewHolder.itemLupin.setVisibility(View.VISIBLE);
                    break;
                case "en:soybeans":
                    viewHolder.itemSoy.setVisibility(View.VISIBLE);
                    break;
                case "en:milk":
                    viewHolder.itemMilk.setVisibility(View.VISIBLE);
                    break;
                case "en:mustard":
                    viewHolder.itemMustard.setVisibility(View.VISIBLE);
                    break;
                case "en:molluscs":
                    viewHolder.itemMolluscs.setVisibility(View.VISIBLE);
                    break;
                case "en:sulphur-dioxide-and-sulphites":
                    viewHolder.itemSulphites.setVisibility(View.VISIBLE);
                    break;
                case "en:nuts":
                    viewHolder.itemNuts.setVisibility(View.VISIBLE);
                    break;
                case "en:peanuts":
                    viewHolder.itemPeanuts.setVisibility(View.VISIBLE);
                    break;
                case "en:eggs":
                    viewHolder.itemEggs.setVisibility(View.VISIBLE);
                    break;
                case "en:sesame-seeds":
                    viewHolder.itemSesame.setVisibility(View.VISIBLE);
                    break;
                case "en:celery":
                    viewHolder.itemCelery.setVisibility(View.VISIBLE);
                    break;
                case "en:fish":
                    viewHolder.itemFish.setVisibility(View.VISIBLE);
                    break;

            }
        }
    }
}

