package edu.uoc.tfg.nutriknow.adapters;

import android.content.res.ColorStateList;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.core.widget.ImageViewCompat;
import androidx.recyclerview.widget.RecyclerView;

import java.util.List;
import java.util.Locale;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.Allergens;
import edu.uoc.tfg.nutriknow.model.Favorite;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.model.Search;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;
import edu.uoc.tfg.nutriknow.views.fragments.SearchesFragment;

/**
 * Adapter for managing the list of searches
 */
public class SearchesRecyclerAdapter extends RecyclerView.Adapter<SearchesRecyclerAdapter.ViewHolder> {

    private static final String ES = "es";

    private final List<Search> searches;
    private final List<Product> products;
    private final List<Favorite> favorites;
    private final WelcomeActivity welcomeActivity;
    private final SearchesFragment searchesFragment;

    /**
     * Adapter constructor
     *
     * @param searches         source list
     * @param products         products list to get allergens content
     * @param welcomeActivity  parent activity
     * @param searchesFragment fragment where the adapter is used
     * @param favorites        favorite products list
     */
    public SearchesRecyclerAdapter(List<Search> searches, List<Product> products,
                                   WelcomeActivity welcomeActivity, SearchesFragment searchesFragment,
                                   List<Favorite> favorites) {
        this.searches = searches;
        this.products = products;
        this.welcomeActivity = welcomeActivity;
        this.searchesFragment = searchesFragment;
        this.favorites = favorites;
    }

    /**
     * ViewHolder structure
     */
    class ViewHolder extends RecyclerView.ViewHolder {

        public TextView itemTitle;
        public TextView itemDetail;
        public ImageView itemCelery;
        public ImageView itemCrustaceans;
        public ImageView itemEggs;
        public ImageView itemFish;
        public ImageView itemGluten;
        public ImageView itemLupin;
        public ImageView itemMilk;
        public ImageView itemMolluscs;
        public ImageView itemMustard;
        public ImageView itemNuts;
        public ImageView itemPeanuts;
        public ImageView itemSesame;
        public ImageView itemSoy;
        public ImageView itemSulphites;

        public ImageView deleteButton;
        public ImageView addFavoriteButton;
        public ImageView newIcon;

        private int selectedIndex;
        private Search selectedSearch;
        private boolean isOtherAction = false;

        public ViewHolder(View itemView) {
            super(itemView);
            // Reference the views in the layout
            itemTitle = (TextView) itemView.findViewById(R.id.item_title);
            itemDetail = (TextView) itemView.findViewById(R.id.item_detail);
            // Icons for pointing out the presence of allergens
            itemCelery = (ImageView) itemView.findViewById(R.id.ic_celery);
            itemCrustaceans = (ImageView) itemView.findViewById(R.id.ic_crustaceans);
            itemEggs = (ImageView) itemView.findViewById(R.id.ic_eggs);
            itemFish = (ImageView) itemView.findViewById(R.id.ic_fish);
            itemGluten = (ImageView) itemView.findViewById(R.id.ic_gluten);
            itemLupin = (ImageView) itemView.findViewById(R.id.ic_lupin);
            itemMilk = (ImageView) itemView.findViewById(R.id.ic_milk);
            itemMolluscs = (ImageView) itemView.findViewById(R.id.ic_molluscs);
            itemMustard = (ImageView) itemView.findViewById(R.id.ic_mustard);
            itemNuts = (ImageView) itemView.findViewById(R.id.ic_nuts);
            itemPeanuts = (ImageView) itemView.findViewById(R.id.ic_peanuts);
            itemSesame = (ImageView) itemView.findViewById(R.id.ic_sesame);
            itemSoy = (ImageView) itemView.findViewById(R.id.ic_soy);
            itemSulphites = (ImageView) itemView.findViewById(R.id.ic_sulphites);

            // Define the behaviour of the delete icon
            deleteButton = (ImageView) itemView.findViewById(R.id.ic_delete);
            deleteButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    isOtherAction = true;
                    itemView.callOnClick();
                    deleteSearch();
                }
            });
            // Define the behaviour of the add icon
            addFavoriteButton = (ImageView) itemView.findViewById(R.id.ic_add_favorite);
            addFavoriteButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    isOtherAction = true;
                    itemView.callOnClick();
                    toggleAddButton(addFavoriteButton, selectedIndex);
                    isOtherAction = false;
                }
            });

            // Define the behaviour of tapping the card in the layout
            itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    selectedIndex = getAdapterPosition();
                    selectedSearch = searches.get(selectedIndex);
                    if (!isOtherAction) {
                        searchesFragment.retrieveProductInformation(searches.get(selectedIndex).getProductCode());
                    }
                }
            });
            // Icons for pointing out a new product
            newIcon = (ImageView) itemView.findViewById(R.id.ic_is_new);
        }

        /**
         * Delete search from the list (product, search and favorite)
         */
        private void deleteSearch() {
            isOtherAction = false;
            searchesFragment.deleteProductInformation(searches.get(selectedIndex).getProductCode(), selectedIndex);
        }
    }

    /**
     * Create the view
     *
     * @param viewGroup card layout to inflate
     * @param viewType  type of view
     * @return viewHolder with the inflated card
     */
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup,
                                         int viewType) {
        View v = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.card_layout, viewGroup, false);
        ViewHolder viewHolder = new ViewHolder(v);
        return viewHolder;
    }

    /**
     * Define the content of the viewHolder
     *
     * @param viewHolder   structure to fill
     * @param listPosition position on the source list
     */
    @Override
    public void onBindViewHolder(final ViewHolder viewHolder, final int listPosition) {
        // Depending on the locale, inform the name of the product
        Locale currentLocale = welcomeActivity.getResources().getConfiguration().locale;
        if (ES.equals(currentLocale.getLanguage())) {
            if (searches.get(listPosition).getProductName_es() != null && !searches.get(listPosition).getProductName_es().isEmpty()) {
                viewHolder.itemTitle.setText(searches.get(listPosition).getProductName_es());
            } else {
                if (searches.get(listPosition).getProductName_en() != null && !searches.get(listPosition).getProductName_en().isEmpty()) {
                    viewHolder.itemTitle.setText(searches.get(listPosition).getProductName_en());
                } else {
                    viewHolder.itemTitle.setText(searchesFragment.getContext().getString(R.string.empty_product_name));
                }

            }
        } else {
            if (searches.get(listPosition).getProductName_en() != null && !searches.get(listPosition).getProductName_en().isEmpty()) {
                viewHolder.itemTitle.setText(searches.get(listPosition).getProductName_en());
            } else {
                if (searches.get(listPosition).getProductName_es() != null && !searches.get(listPosition).getProductName_es().isEmpty()) {
                    viewHolder.itemTitle.setText(searches.get(listPosition).getProductName_es());
                } else {
                    viewHolder.itemTitle.setText(searchesFragment.getContext().getString(R.string.empty_product_name));
                }

            }
        }
        // Set the barcode content
        viewHolder.itemDetail.setText(searches.get(listPosition).getProductCode());
        // Define the add icon appareance and behaviour
        boolean isFavorite = false;
        for (Favorite favorite : favorites) {
            if (searches.get(listPosition).getProductCode().equals(favorite.getProductCode())) {
                isFavorite = true;
                break;
            }
        }
        if (isFavorite) {
            viewHolder.addFavoriteButton.setImageDrawable(
                    welcomeActivity.getDrawable(R.drawable.ic_menu_my_favorites));
        } else {
            viewHolder.addFavoriteButton.setImageDrawable(
                    welcomeActivity.getDrawable(R.drawable.ic_menu_my_favorites_full));
        }
        ImageViewCompat.setImageTintList(viewHolder.addFavoriteButton,
                ColorStateList.valueOf(welcomeActivity.getColor(R.color.textColorPrimary)));

        // Display only the icons for the allergens found in the product
        hideAllIcons(viewHolder);
        displayAllergenIcons(viewHolder, products.get(listPosition).getIngredientsData().getAllergens_es());
        // Toggle the visibility of the new flag
        if (searches.get(listPosition).isNew()) {
            viewHolder.newIcon.setVisibility(View.VISIBLE);
        } else {
            viewHolder.newIcon.setVisibility(View.GONE);
        }
    }

    /**
     * Get the total size
     *
     * @return size
     */
    @Override
    public int getItemCount() {
        return searches.size();
    }

    /**
     * Toggle add button
     *
     * @param addFavoriteButton view to toggle
     */
    private void toggleAddButton(ImageView addFavoriteButton, int listPosition) {
        boolean isFavorite = false;
        // Toggle functionality if the product is already in the favorite products list
        for (Favorite favorite : favorites) {
            if (searches.get(listPosition).getProductCode().equals(favorite.getProductCode())) {
                isFavorite = true;
                break;
            }
        }
        if (isFavorite) {
            // Delete product from the favorite list
            searchesFragment.deleteProductFromFavorites(products.get(listPosition).getSummaryData().getCode());
            addFavoriteButton.setImageDrawable(
                    welcomeActivity.getDrawable(R.drawable.ic_menu_my_favorites));
        } else {
            // Add product to the favorite list
            searchesFragment.addProductToFavorites(products.get(listPosition));
            addFavoriteButton.setImageDrawable(
                    welcomeActivity.getDrawable(R.drawable.ic_menu_my_favorites_full));
        }
    }

    /**
     * Hide all the icons in the viewHolder
     *
     * @param viewHolder structure where hide the icons
     */
    private void hideAllIcons(ViewHolder viewHolder) {
        viewHolder.itemGluten.setVisibility(View.GONE);
        viewHolder.itemCrustaceans.setVisibility(View.GONE);
        viewHolder.itemLupin.setVisibility(View.GONE);
        viewHolder.itemSoy.setVisibility(View.GONE);
        viewHolder.itemMilk.setVisibility(View.GONE);
        viewHolder.itemMustard.setVisibility(View.GONE);
        viewHolder.itemMolluscs.setVisibility(View.GONE);
        viewHolder.itemSulphites.setVisibility(View.GONE);
        viewHolder.itemNuts.setVisibility(View.GONE);
        viewHolder.itemPeanuts.setVisibility(View.GONE);
        viewHolder.itemEggs.setVisibility(View.GONE);
        viewHolder.itemSesame.setVisibility(View.GONE);
        viewHolder.itemCelery.setVisibility(View.GONE);
        viewHolder.itemFish.setVisibility(View.GONE);
    }

    /**
     * Display the icons of the allergens contained in the product
     *
     * @param viewHolder structure where display the icons
     * @param allergens  list of allergens found
     */
    private void displayAllergenIcons(ViewHolder viewHolder, List<Allergens.Allergen> allergens) {
        for (Allergens.Allergen allergen : allergens) {
            switch (allergen.id) {
                case "en:gluten":
                    viewHolder.itemGluten.setVisibility(View.VISIBLE);
                    break;
                case "en:crustaceans":
                    viewHolder.itemCrustaceans.setVisibility(View.VISIBLE);
                    break;
                case "en:lupin":
                    viewHolder.itemLupin.setVisibility(View.VISIBLE);
                    break;
                case "en:soybeans":
                    viewHolder.itemSoy.setVisibility(View.VISIBLE);
                    break;
                case "en:milk":
                    viewHolder.itemMilk.setVisibility(View.VISIBLE);
                    break;
                case "en:mustard":
                    viewHolder.itemMustard.setVisibility(View.VISIBLE);
                    break;
                case "en:molluscs":
                    viewHolder.itemMolluscs.setVisibility(View.VISIBLE);
                    break;
                case "en:sulphur-dioxide-and-sulphites":
                    viewHolder.itemSulphites.setVisibility(View.VISIBLE);
                    break;
                case "en:nuts":
                    viewHolder.itemNuts.setVisibility(View.VISIBLE);
                    break;
                case "en:peanuts":
                    viewHolder.itemPeanuts.setVisibility(View.VISIBLE);
                    break;
                case "en:eggs":
                    viewHolder.itemEggs.setVisibility(View.VISIBLE);
                    break;
                case "en:sesame-seeds":
                    viewHolder.itemSesame.setVisibility(View.VISIBLE);
                    break;
                case "en:celery":
                    viewHolder.itemCelery.setVisibility(View.VISIBLE);
                    break;
                case "en:fish":
                    viewHolder.itemFish.setVisibility(View.VISIBLE);
                    break;

            }
        }
    }
}

