package edu.uoc.tfg.nutriknow.api;

import android.content.Context;
import android.util.Log;

import com.google.gson.Gson;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Executor;
import java.util.regex.Pattern;

import edu.uoc.tfg.nutriknow.api.interfaces.OpenFoodFactsService;
import edu.uoc.tfg.nutriknow.model.Additives;
import edu.uoc.tfg.nutriknow.model.AdditivesClasses;
import edu.uoc.tfg.nutriknow.model.Allergens;
import edu.uoc.tfg.nutriknow.model.Categories;
import edu.uoc.tfg.nutriknow.model.IngredientsAnalysis;
import edu.uoc.tfg.nutriknow.model.Labels;
import edu.uoc.tfg.nutriknow.model.NovaGroups;
import edu.uoc.tfg.nutriknow.model.NutrientsLevel;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AdditivesClassesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AdditivesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.AllergensDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.CategoriesDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.IngredientAnalysisDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.LabelsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.NovaGroupsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.sqlite.adapters.NutrientsLevelsDataBaseAdapter;
import edu.uoc.tfg.nutriknow.utils.DatabaseCatalogUtils;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * Asynchronous repository for getting the catalog information from Open Food Facts api
 */
public class OpenFoodFactsRepository {

    private final Executor executor;
    private final Context ctx;

    private static final String TAG = "OpenFoodFactsRepository";

    private static final String ES = "es";
    private static final String EN = "en";

    public static final String RUNNING = "RUNNING";
    public static final String SUCCESS = "SUCCESS";
    public static final String FAIL = "FAIL";

    private static final Pattern patternStartsWithNumberOrUpper = Pattern.compile("\\b[0-9A-Z].*?\\b");
    private static final Pattern patternStartsWithUppercase = Pattern.compile("\\b[A-Z].*?\\b");

    private AllergensDataBaseAdapter allergensDataBaseAdapter;
    private CategoriesDataBaseAdapter categoriesDataBaseAdapter;
    private LabelsDataBaseAdapter labelsDataBaseAdapter;
    private NovaGroupsDataBaseAdapter novaGroupsDataBaseAdapter;
    private NutrientsLevelsDataBaseAdapter nutrientsLevelsDataBaseAdapter;
    private IngredientAnalysisDataBaseAdapter ingredientAnalysisDataBaseAdapter;
    private AdditivesClassesDataBaseAdapter additivesClassesDataBaseAdapter;
    private AdditivesDataBaseAdapter additivesDataBaseAdapter;

    private OpenFoodFactsService apiServiceEs;
    private OpenFoodFactsService apiServiceEn;
    private OpenFoodFactsService apiServiceWorld;

    private static final Map<String, String[]> results = new HashMap<>();

    /**
     * Repository constructor
     * @param context context
     * @param executor executor
     */
    public OpenFoodFactsRepository(Context context, Executor executor) {
        this.executor = executor;
        this.ctx = context;
    }

    /**
     * Define the database adapters and retrieve data from api
     */
    public void loadCatalogData() {
        executor.execute(new Runnable() {
            @Override
            public void run() {
                allergensDataBaseAdapter = new AllergensDataBaseAdapter(ctx);
                categoriesDataBaseAdapter = new CategoriesDataBaseAdapter(ctx);
                labelsDataBaseAdapter = new LabelsDataBaseAdapter(ctx);
                novaGroupsDataBaseAdapter = new NovaGroupsDataBaseAdapter(ctx);
                nutrientsLevelsDataBaseAdapter = new NutrientsLevelsDataBaseAdapter(ctx);
                ingredientAnalysisDataBaseAdapter = new IngredientAnalysisDataBaseAdapter(ctx);
                additivesClassesDataBaseAdapter = new AdditivesClassesDataBaseAdapter(ctx);
                additivesDataBaseAdapter = new AdditivesDataBaseAdapter(ctx);

                retrieveDataFromRESTApi();
            }
        });
    }

    /**
     * Get state of each request (RUNNING, SUCCESS, FAIL
     * @return map with results
     */
    public Map<String, String[]> getResults() {
        return results;
    }

    /**
     * Get the catalog data from api and store it in database
     */
    private void retrieveDataFromRESTApi() {
        // Define api interfaces
        apiServiceEs = OpenFoodFactsClient.getEsClient().create(OpenFoodFactsService.class);
        apiServiceEn = OpenFoodFactsClient.getEnClient().create(OpenFoodFactsService.class);
        apiServiceWorld = OpenFoodFactsClient.getWorldClient().create(OpenFoodFactsService.class);

        // Retrieve allergens
        List<Allergens.Allergen> allergensES =
                DatabaseCatalogUtils.retrieveAllergensFromLocalDatabase(allergensDataBaseAdapter, ES);
        List<Allergens.Allergen> allergensEN =
                DatabaseCatalogUtils.retrieveAllergensFromLocalDatabase(allergensDataBaseAdapter, EN);
        if (allergensES == null || allergensES.isEmpty() || allergensEN == null || allergensEN.isEmpty()) {
            results.put("A_ES", new String[]{RUNNING});
            getAllergensOpenFoodFactsCall(apiServiceEs, ES);
            results.put("A_EN", new String[]{RUNNING});
            getAllergensOpenFoodFactsCall(apiServiceEn, EN);
        }

        // Retrieve categories
        List<Categories.Category> categoriesES =
                DatabaseCatalogUtils.retrieveCategoriesFromLocalDatabase(categoriesDataBaseAdapter, ES);
        List<Categories.Category> categoriesEN =
                DatabaseCatalogUtils.retrieveCategoriesFromLocalDatabase(categoriesDataBaseAdapter, EN);
        if (categoriesES == null || categoriesES.isEmpty() || categoriesEN == null || categoriesEN.isEmpty()) {
            results.put("C_ES", new String[]{RUNNING});
            getCategoriesOpenFoodFactsCall(apiServiceEs, ES);
            results.put("C_EN", new String[]{RUNNING});
            getCategoriesOpenFoodFactsCall(apiServiceEn, EN);
        }

        // Retrieve labels
        List<Labels.Label> labelsES =
                DatabaseCatalogUtils.retrieveLabelsFromLocalDatabase(labelsDataBaseAdapter, ES);
        List<Labels.Label> labelsEN =
                DatabaseCatalogUtils.retrieveLabelsFromLocalDatabase(labelsDataBaseAdapter, EN);
        if (labelsES == null || labelsES.isEmpty() || labelsEN == null || labelsEN.isEmpty()) {
            results.put("L_ES", new String[]{RUNNING});
            getLabelsOpenFoodFactsCall(apiServiceEs, ES);
            results.put("L_EN", new String[]{RUNNING});
            getLabelsOpenFoodFactsCall(apiServiceEn, EN);
        }

        // Retrieve ingredients analysis
        List<IngredientsAnalysis.IngredientAnalysis> ingredientsAnalysisES =
                DatabaseCatalogUtils.retrieveIngredientsAnalysisFromLocalDatabase(ingredientAnalysisDataBaseAdapter, ES);
        List<IngredientsAnalysis.IngredientAnalysis> ingredientsAnalysisEN =
                DatabaseCatalogUtils.retrieveIngredientsAnalysisFromLocalDatabase(ingredientAnalysisDataBaseAdapter, EN);
        if (ingredientsAnalysisES == null || ingredientsAnalysisES.isEmpty() ||
                ingredientsAnalysisEN == null || ingredientsAnalysisEN.isEmpty()) {
            results.put("IA_ES", new String[]{RUNNING});
            getIngredientsAnalysisOpenFoodFactsCall(apiServiceEs, ES);
            results.put("IA_EN", new String[]{RUNNING});
            getIngredientsAnalysisOpenFoodFactsCall(apiServiceEn, EN);
        }

        // Retrieve additives classes
        AdditivesClasses additivesClasses =
                DatabaseCatalogUtils.retrieveAdditivesClassesFromLocalDatabase(additivesClassesDataBaseAdapter);
        if (additivesClasses == null || additivesClasses.getEnAcid().code == null) {
            results.put("AC", new String[]{RUNNING});
            getAdditivesClassesOpenFoodFactsCall(apiServiceWorld);
        }

        // Retrieve additives
        List<Additives> additives =
                DatabaseCatalogUtils.retrieveAdditivesFromLocalDatabase(additivesDataBaseAdapter);
        if (additives == null || additives.isEmpty()) {
            results.put("AD", new String[]{RUNNING});
            getAdditivesOpenFoodFactsCall(apiServiceWorld);
        }

        // Retrieve nova groups
        NovaGroups novaGroups =
                DatabaseCatalogUtils.retrieveNovaGroupsFromLocalDatabase(novaGroupsDataBaseAdapter);
        if (novaGroups == null || novaGroups.getEn1UnprocessedOrMinimallyProcessedFoods().code == null) {
            results.put("NG", new String[]{RUNNING});
            getNovaGroupsOpenFoodFactsCall(apiServiceWorld);
        }

        // Retrieve nutrients Levels
        NutrientsLevel nutrientsLevel =
                DatabaseCatalogUtils.retrieveNutrientsLevelsFromLocalDatabase(nutrientsLevelsDataBaseAdapter);
        if (nutrientsLevel == null || nutrientsLevel.getEnFatInHighQuantity().code == null) {
            results.put("NL", new String[]{RUNNING});
            getNutrientLevelsOpenFoodFactsCall(apiServiceWorld);
        }
    }

    /**
     * Get allergens api rest call and database storage
     * @param apiService endpoint
     * @param language selected locale
     */
    private void getAllergensOpenFoodFactsCall(OpenFoodFactsService apiService, String language) {
        Call<Allergens> call = apiService.getAllAllergens();
        call.enqueue(new Callback<Allergens>() {
            @Override
            public void onResponse(Call<Allergens> call, Response<Allergens> response) {
                Allergens resource = response.body();
                List<Allergens.Allergen> allergens = new ArrayList<>();
                for (Allergens.Allergen allergen : resource.tags) {
                    if (patternStartsWithUppercase.matcher(allergen.name).matches()) {
                        allergens.add(allergen);
                    }
                }
                Collections.sort(allergens, (s1, s2) -> s1.name.compareTo(s2.name));
                DatabaseCatalogUtils.saveAllergensInLocalDatabase(allergensDataBaseAdapter, allergens, language);
                results.put("A_" + language.toUpperCase(), new String[]{SUCCESS, String.valueOf(resource.count), String.valueOf(allergens.size())});
            }

            @Override
            public void onFailure(Call<Allergens> call, Throwable t) {
                call.cancel();
                Log.e(TAG, t.getMessage() + " -> getAllergensOpenFoodFactsCall");
                results.put("A_" + language.toUpperCase(), new String[]{FAIL});
            }
        });
    }

    /**
     * Get categories api rest call and database storage
     * @param apiService endpoint
     * @param language selected locale
     */
    private void getCategoriesOpenFoodFactsCall(OpenFoodFactsService apiService, String language) {
        Call<Categories> call = apiService.getAllCategories();
        call.enqueue(new Callback<Categories>() {
            @Override
            public void onResponse(Call<Categories> call, Response<Categories> response) {
                Categories resource = response.body();

                List<Categories.Category> categories = new ArrayList<>();
                for (Categories.Category category : resource.tags) {
                    if (patternStartsWithNumberOrUpper.matcher(category.name).matches()) {
                        categories.add(category);
                    }
                }
                Collections.sort(categories, (s1, s2) -> s1.name.compareTo(s2.name));
                DatabaseCatalogUtils.saveCategoriesInLocalDatabase(categoriesDataBaseAdapter, categories, language);
                results.put("C_" + language.toUpperCase(), new String[]{SUCCESS, String.valueOf(resource.count), String.valueOf(categories.size())});
            }

            @Override
            public void onFailure(Call<Categories> call, Throwable t) {
                call.cancel();
                Log.e(TAG, t.getMessage() + " -> getCategoriesOpenFoodFactsCall");
                results.put("C_" + language.toUpperCase(), new String[]{FAIL});
            }
        });
    }

    /**
     * Get labels api rest call and database storage
     * @param apiService endpoint
     * @param language selected locale
     */
    private void getLabelsOpenFoodFactsCall(OpenFoodFactsService apiService, String language) {
        Call<Labels> call = apiService.getAllLabels();
        call.enqueue(new Callback<Labels>() {
            @Override
            public void onResponse(Call<Labels> call, Response<Labels> response) {
                Labels resource = response.body();
                List<Labels.Label> labels = new ArrayList<>();
                for (Labels.Label label : resource.tags) {
                    if (patternStartsWithNumberOrUpper.matcher(label.name).matches()) {
                        labels.add(label);
                    }
                }
                Collections.sort(labels, (s1, s2) -> s1.name.compareTo(s2.name));
                DatabaseCatalogUtils.saveLabelsInLocalDatabase(labelsDataBaseAdapter, labels, language);
                results.put("L_" + language.toUpperCase(), new String[]{SUCCESS, String.valueOf(resource.count), String.valueOf(labels.size())});
            }

            @Override
            public void onFailure(Call<Labels> call, Throwable t) {
                call.cancel();
                Log.e(TAG, t.getMessage() + " -> getLabelsOpenFoodFactsCall");
                results.put("L_" + language.toUpperCase(), new String[]{FAIL});
            }
        });
    }

    /**
     * Get ingredients analysis api rest call and database storage
     * @param apiService endpoint
     * @param language selected locale
     */
    private void getIngredientsAnalysisOpenFoodFactsCall(OpenFoodFactsService apiService, String language) {
        Call<IngredientsAnalysis> call = apiService.getAllIngredientAnalysis();
        call.enqueue(new Callback<IngredientsAnalysis>() {
            @Override
            public void onResponse(Call<IngredientsAnalysis> call, Response<IngredientsAnalysis> response) {
                IngredientsAnalysis resource = response.body();
                DatabaseCatalogUtils.saveIngredientsAnalysisInLocalDatabase(ingredientAnalysisDataBaseAdapter, resource.tags, language);
                results.put("IA_" + language.toUpperCase(), new String[]{SUCCESS, String.valueOf(resource.count), String.valueOf(resource.count)});
            }

            @Override
            public void onFailure(Call<IngredientsAnalysis> call, Throwable t) {
                call.cancel();
                Log.e(TAG, t.getMessage() + " -> getIngredientsAnalysisOpenFoodFactsCall");
                results.put("IA_" + language.toUpperCase(), new String[]{FAIL});
            }
        });
    }

    /**
     * Get additives api rest call and database storage
     * @param apiService endpoint
     */
    private void getAdditivesOpenFoodFactsCall(OpenFoodFactsService apiService) {
        Call<JsonObject> call = apiService.getAllAdditives();
        call.enqueue(new Callback<JsonObject>() {
            @Override
            public void onResponse(Call<JsonObject> call, Response<JsonObject> response) {
                JsonParser parser = new JsonParser();
                JsonElement element = parser.parse(response.body().toString());
                JsonObject obj = element.getAsJsonObject();
                List<Additives> additives = new ArrayList<>();
                // Parse api response
                Set<Map.Entry<String, JsonElement>> entries = obj.entrySet();
                for (Map.Entry<String, JsonElement> entry : entries) {
                    Additives additive = new Additives();
                    additive.code = entry.getKey();
                    JsonObject content = new Gson().fromJson(entry.getValue().getAsJsonObject(), JsonObject.class);
                    if (content.getAsJsonObject("name") != null) {
                        JsonObject name = content.getAsJsonObject("name");
                        if(name.get("es") != null){
                            additive.name_es = (name.get("es")).toString().replace("\"", "");
                        }
                        if(name.get("en") != null){
                            additive.name_en = (name.get("en")).toString().replace("\"", "");
                        }
                    }
                    // Parse additive classes codes
                    if (content.getAsJsonObject("additives_classes") != null) {
                        JsonObject classes = content.getAsJsonObject("additives_classes");
                        String classesStr = classes.toString().replace("\"","");
                        classesStr = classesStr.replace("{", "").replace("}", "");
                        String[] arrayClasses = classesStr.split(",");
                        additive.additives_classes_tags = Arrays.asList(arrayClasses);
                    }
                    additives.add(additive);
                }
                DatabaseCatalogUtils.saveAdditivesInLocalDatabase(additivesDataBaseAdapter, additives);
                results.put("AD", new String[]{SUCCESS, String.valueOf(additives.size()), String.valueOf(additives.size())});
            }

            @Override
            public void onFailure(Call<JsonObject> call, Throwable t) {
                call.cancel();
                Log.e(TAG, t.getMessage() + " -> getAdditivesOpenFoodFactsCall");
                results.put("AD", new String[]{FAIL});
            }
        });
    }

    /**
     * Get additives clases api rest call and database storage
     * @param apiService endpoint
     */
    private void getAdditivesClassesOpenFoodFactsCall(OpenFoodFactsService apiService) {
        Call<AdditivesClasses> call = apiService.getAllAdditivesClasses();
        call.enqueue(new Callback<AdditivesClasses>() {
            @Override
            public void onResponse(Call<AdditivesClasses> call, Response<AdditivesClasses> response) {
                AdditivesClasses resource = response.body();
                DatabaseCatalogUtils.saveAdditivesClassesInLocalDatabase(additivesClassesDataBaseAdapter, resource);
                results.put("AC", new String[]{SUCCESS, String.valueOf(55), String.valueOf(55)});
            }

            @Override
            public void onFailure(Call<AdditivesClasses> call, Throwable t) {
                call.cancel();
                Log.e(TAG, t.getMessage() + " -> getAdditivesOpenFoodFactsCall");
                results.put("AC", new String[]{FAIL});
            }
        });
    }

    /**
     * Get nova groups api rest call and database storage
     * @param apiService endpoint
     */
    private void getNovaGroupsOpenFoodFactsCall(OpenFoodFactsService apiService) {
        Call<NovaGroups> call = apiService.getAllNovaGroups();
        call.enqueue(new Callback<NovaGroups>() {
            @Override
            public void onResponse(Call<NovaGroups> call, Response<NovaGroups> response) {
                NovaGroups resource = response.body();
                DatabaseCatalogUtils.saveNovaGroupsInLocalDatabase(novaGroupsDataBaseAdapter, resource);
                results.put("NG", new String[]{SUCCESS, String.valueOf(5), String.valueOf(5)});
            }

            @Override
            public void onFailure(Call<NovaGroups> call, Throwable t) {
                call.cancel();
                Log.e(TAG, t.getMessage() + " -> getNovaGroupsOpenFoodFactsCall");
                results.put("NG", new String[]{FAIL});
            }
        });
    }

    /**
     * Get nutrients levels api rest call and database storage
     * @param apiService endpoint
     */
    private void getNutrientLevelsOpenFoodFactsCall(OpenFoodFactsService apiService) {
        Call<NutrientsLevel> call = apiService.getAllNutrientLevels();
        call.enqueue(new Callback<NutrientsLevel>() {
            @Override
            public void onResponse(Call<NutrientsLevel> call, Response<NutrientsLevel> response) {
                NutrientsLevel resource = response.body();
                DatabaseCatalogUtils.saveNutrientsLevelsInLocalDatabase(nutrientsLevelsDataBaseAdapter, resource);
                results.put("NL", new String[]{SUCCESS, String.valueOf(12), String.valueOf(12)});
            }

            @Override
            public void onFailure(Call<NutrientsLevel> call, Throwable t) {
                call.cancel();
                Log.e(TAG, t.getMessage() + " -> getNovaGroupsOpenFoodFactsCall");
                results.put("NL", new String[]{FAIL});
            }
        });
    }
}
