package edu.uoc.tfg.nutriknow.sqlite.adapters;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import edu.uoc.tfg.nutriknow.model.Additives;
import edu.uoc.tfg.nutriknow.sqlite.DataBaseHelper;

/**
 * Database adapter for additives
 */
public class AdditivesDataBaseAdapter extends DataBaseAdapter {

    private static final String TAG = "AdditivesAdapter";

    static final int DATABASE_VERSION = 1;

    // SQL Statement to create a new database.
    public static final String TABLE_NAME = "ADDITIVE";
    public static final String CREATE_TABLE = "CREATE TABLE IF NOT EXISTS ADDITIVE (ID integer primary key autoincrement, CODE text, NAME_ES text, NAME_EN text, ADDITIVE_CLASSES text); ";
    public static final String DROP_TABLE = "DROP TABLE IF EXISTS ADDITIVE";

    // SQL Columns
    static final String CODE_COLUMN = "CODE";
    static final String NAME_ES_COLUMN = "NAME_ES";
    static final String NAME_EN_COLUMN = "NAME_EN";
    static final String ADDITIVE_CLASSES_COLUMN = "ADDITIVE_CLASSES";

    /**
     * Adapter constructor
     *
     * @param _context context
     */
    public AdditivesDataBaseAdapter(Context _context) {
        context = _context;
        dbHelper = new DataBaseHelper(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    /**
     * Method to open the Database
     *
     * @return adapter
     * @throws SQLException
     */
    public AdditivesDataBaseAdapter open() throws SQLException {
        db = dbHelper.getWritableDatabase();
        return this;
    }

    /**
     * Method to close the Database
     */
    public void close() {
        db.close();
    }

    /**
     * Method to insert a record in Table
     *
     * @param additivesList source
     * @return boolean
     */
    public boolean insertAdditives(List<Additives> additivesList) {
        boolean result = true;
        long count = 0;
        try {
            ContentValues newValues = new ContentValues();
            for (Additives additives : additivesList) {
                // Assign values for each column.
                newValues.put(CODE_COLUMN, additives.code);
                newValues.put(NAME_ES_COLUMN, additives.name_es);
                newValues.put(NAME_EN_COLUMN, additives.name_en);
                String additives_classes_tag = "";
                if (additives.additives_classes_tags != null && !additives.additives_classes_tags.isEmpty()) {
                    for (String tag : additives.additives_classes_tags) {
                        additives_classes_tag = additives_classes_tag + "," + tag.trim();
                    }
                    additives_classes_tag = additives_classes_tag.substring(1);
                }
                newValues.put(ADDITIVE_CLASSES_COLUMN, additives_classes_tag);
                // Insert the row into your table
                db = dbHelper.getWritableDatabase();
                count = db.insert(TABLE_NAME, null, newValues);

            }
            Log.i(TAG, "Inserted additives -> " + String.valueOf(count));
        } catch (Exception ex) {
            result = false;
            Log.e(TAG, "Error in AdditivesDataBaseAdapter.insertAdditives" + ex.getMessage());
        } finally {
            this.close();
        }
        return result;
    }

    /**
     * Method to get the contents
     *
     * @return additives content
     */
    public List<Additives> getAllAdditives() {
        List<Additives> additivesList = new ArrayList<>();
        Cursor cursor = null;
        try {
            db = dbHelper.getReadableDatabase();
            cursor = db.query(TABLE_NAME, null, null, null, null, null, null);

            // Move to first row
            if (!cursor.moveToFirst())
                return additivesList;
            do {
                Additives additives = new Additives();
                additives.code = cursor.getString(cursor.getColumnIndex(CODE_COLUMN));
                additives.name_es = cursor.getString(cursor.getColumnIndex(NAME_ES_COLUMN));
                additives.name_en = cursor.getString(cursor.getColumnIndex(NAME_EN_COLUMN));
                String classes = cursor.getString(cursor.getColumnIndex(ADDITIVE_CLASSES_COLUMN));
                additives.additives_classes_tags = Arrays.asList(classes.replace("en:en:", "en:").split(","));
                additivesList.add(additives);
            } while (cursor.moveToNext());
        } finally {
            cursor.close();
            this.close();
        }
        return additivesList;
    }
}
