package edu.uoc.tfg.nutriknow.sqlite.adapters;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.tfg.nutriknow.model.Allergens;
import edu.uoc.tfg.nutriknow.sqlite.DataBaseHelper;

/**
 * Database adapter for allergens
 */
public class AllergensDataBaseAdapter extends DataBaseAdapter {

    private static final String TAG = "AllergensAdapter";

    static final int DATABASE_VERSION = 1;

    // SQL Statement to create a new database.
    public static final String TABLE_NAME = "ALLERGEN";
    public static final String CREATE_TABLE = "CREATE TABLE IF NOT EXISTS ALLERGEN (ID integer primary key autoincrement, LANGUAGE text, CODE text, NAME text, PRODUCTS integer, KNOWN integer, URL text); ";
    public static final String DROP_TABLE = "DROP TABLE IF EXISTS ALLERGEN";

    // SQL Columns
    static final String LANGUAGE_COLUMN = "LANGUAGE";
    static final String CODE_COLUMN = "CODE";
    static final String NAME_COLUMN = "NAME";
    static final String PRODUCTS_COLUMN = "PRODUCTS";
    static final String KNOWN_COLUMN = "KNOWN";
    static final String URL_COLUMN = "URL";

    private static final String WHERE = "LANGUAGE=?";

    /**
     * Adapter constructor
     *
     * @param _context context
     */
    public AllergensDataBaseAdapter(Context _context) {
        context = _context;
        dbHelper = new DataBaseHelper(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    /**
     * Method to open the Database
     *
     * @return adapter
     * @throws SQLException
     */
    public AllergensDataBaseAdapter open() throws SQLException {
        db = dbHelper.getWritableDatabase();
        return this;
    }

    /**
     * Method to close the Database
     */
    public void close() {
        db.close();
    }

    /**
     * Method to insert a record in Table
     *
     * @param allergenList source
     * @param language     selected locale
     * @return boolean
     */
    public boolean insertAllergens(List<Allergens.Allergen> allergenList, String language) {
        boolean result = true;
        long count = 0;
        try {
            ContentValues newValues = new ContentValues();
            for (Allergens.Allergen allergen : allergenList) {
                // Assign values for each column.
                newValues.put(LANGUAGE_COLUMN, language);
                newValues.put(CODE_COLUMN, allergen.id);
                newValues.put(NAME_COLUMN, allergen.name);
                newValues.put(PRODUCTS_COLUMN, allergen.products);
                newValues.put(KNOWN_COLUMN, allergen.known);
                newValues.put(URL_COLUMN, allergen.url);
                // Insert the row into your table
                db = dbHelper.getWritableDatabase();
                count = db.insert(TABLE_NAME, null, newValues);

            }
            Log.i(TAG, "Inserted allergens -> " + String.valueOf(count));
        } catch (Exception ex) {
            result = false;
            Log.e(TAG, "Error in AllergensDataBaseAdapter.insertAllergens" + ex.getMessage());
        } finally {
            this.close();
        }
        return result;
    }

    /**
     * Method to get the contents
     *
     * @param language selected locale
     * @return allergens content
     */
    public List<Allergens.Allergen> getAllergensByLanguage(String language) {
        List<Allergens.Allergen> allergenList = new ArrayList<>();
        Cursor cursor = null;
        try {
            db = dbHelper.getReadableDatabase();
            cursor = db.query(TABLE_NAME, null, WHERE, new String[]{language}, null, null, null);

            // Move to first row
            if (!cursor.moveToFirst())
                return allergenList;
            do {
                Allergens.Allergen allergen = new Allergens.Allergen();
                allergen.id = cursor.getString(cursor.getColumnIndex(CODE_COLUMN));
                allergen.name = cursor.getString(cursor.getColumnIndex(NAME_COLUMN));
                allergen.products = cursor.getInt(cursor.getColumnIndex(PRODUCTS_COLUMN));
                allergen.known = cursor.getInt(cursor.getColumnIndex(KNOWN_COLUMN));
                allergen.url = cursor.getString(cursor.getColumnIndex(URL_COLUMN));
                allergenList.add(allergen);
            } while (cursor.moveToNext());
        } finally {
            cursor.close();
            this.close();
        }
        return allergenList;
    }
}
