package edu.uoc.tfg.nutriknow.sqlite.adapters;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.tfg.nutriknow.model.Categories;
import edu.uoc.tfg.nutriknow.sqlite.DataBaseHelper;

/**
 * Database adapter for categories
 */
public class CategoriesDataBaseAdapter extends DataBaseAdapter {

    private static final String TAG = "CategoriesAdapter";

    static final int DATABASE_VERSION = 1;

    // SQL Statement to create a new database.
    public static final String TABLE_NAME = "CATEGORY";
    public static final String CREATE_TABLE = "CREATE TABLE IF NOT EXISTS CATEGORY (ID integer primary key autoincrement, LANGUAGE text, CODE text, NAME text, PRODUCTS integer, KNOWN integer, URL text); ";
    public static final String DROP_TABLE = "DROP TABLE IF EXISTS CATEGORY";

    // SQL Columns
    static final String LANGUAGE_COLUMN = "LANGUAGE";
    static final String CODE_COLUMN = "CODE";
    static final String NAME_COLUMN = "NAME";
    static final String PRODUCTS_COLUMN = "PRODUCTS";
    static final String KNOWN_COLUMN = "KNOWN";
    static final String URL_COLUMN = "URL";

    private static final String WHERE = "LANGUAGE=?";

    /**
     * Adapter constructor
     *
     * @param _context context
     */
    public CategoriesDataBaseAdapter(Context _context) {
        context = _context;
        dbHelper = new DataBaseHelper(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    /**
     * Method to open the Database
     *
     * @return adapter
     * @throws SQLException
     */
    public CategoriesDataBaseAdapter open() throws SQLException {
        db = dbHelper.getWritableDatabase();
        return this;
    }

    /**
     * Method to close the Database
     */
    public void close() {
        db.close();
    }

    /**
     * Method to insert a record in Table
     *
     * @param categoriesList source
     * @param language       selected locale
     * @return boolean
     */
    public boolean insertCategories(List<Categories.Category> categoriesList, String language) {
        boolean result = true;
        long count = 0;
        try {
            ContentValues newValues = new ContentValues();
            for (Categories.Category category : categoriesList) {
                // Assign values for each column.
                newValues.put(LANGUAGE_COLUMN, language);
                newValues.put(CODE_COLUMN, category.id);
                newValues.put(NAME_COLUMN, category.name);
                newValues.put(PRODUCTS_COLUMN, category.products);
                newValues.put(KNOWN_COLUMN, category.known);
                newValues.put(URL_COLUMN, category.url);
                // Insert the row into your table
                db = dbHelper.getWritableDatabase();
                count = db.insert(TABLE_NAME, null, newValues);

            }
            Log.i(TAG, "Inserted categories -> " + String.valueOf(count));
        } catch (Exception ex) {
            result = false;
            Log.e(TAG, "Error in CategoriesDataBaseAdapter.insertCategories" + ex.getMessage());
        } finally {
            this.close();
        }
        return result;
    }

    /**
     * Method to get the contents
     *
     * @param language selected locale
     * @return categories content
     */
    public List<Categories.Category> getCategoriesByLanguage(String language) {
        List<Categories.Category> categoriesList = new ArrayList<>();
        Cursor cursor = null;
        try {
            db = dbHelper.getReadableDatabase();
            cursor = db.query(TABLE_NAME, null, WHERE, new String[]{language}, null, null, null);

            // Move to first row
            if (!cursor.moveToFirst())
                return categoriesList;
            do {
                Categories.Category category = new Categories.Category();
                category.id = cursor.getString(cursor.getColumnIndex(CODE_COLUMN));
                category.name = cursor.getString(cursor.getColumnIndex(NAME_COLUMN));
                category.products = cursor.getInt(cursor.getColumnIndex(PRODUCTS_COLUMN));
                category.known = cursor.getInt(cursor.getColumnIndex(KNOWN_COLUMN));
                category.url = cursor.getString(cursor.getColumnIndex(URL_COLUMN));
                categoriesList.add(category);
            } while (cursor.moveToNext());
        } finally {
            cursor.close();
            this.close();
        }
        return categoriesList;
    }
}
