package edu.uoc.tfg.nutriknow.sqlite.adapters;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.tfg.nutriknow.model.IngredientsAnalysis;
import edu.uoc.tfg.nutriknow.sqlite.DataBaseHelper;

/**
 * Database adapter for ingredients analysis
 */
public class IngredientAnalysisDataBaseAdapter extends DataBaseAdapter {

    private static final String TAG = "IngredientAnalysisDBA";

    static final int DATABASE_VERSION = 1;

    // SQL Statement to create a new database.
    public static final String TABLE_NAME = "INGREDIENT_ANALYSIS";
    public static final String CREATE_TABLE = "CREATE TABLE IF NOT EXISTS INGREDIENT_ANALYSIS (ID integer primary key autoincrement, LANGUAGE text, CODE text, NAME text, PRODUCTS integer, KNOWN integer, URL text); ";
    public static final String DROP_TABLE = "DROP TABLE IF EXISTS INGREDIENT_ANALYSIS";

    // SQL Columns
    static final String LANGUAGE_COLUMN = "LANGUAGE";
    static final String CODE_COLUMN = "CODE";
    static final String NAME_COLUMN = "NAME";
    static final String PRODUCTS_COLUMN = "PRODUCTS";
    static final String KNOWN_COLUMN = "KNOWN";
    static final String URL_COLUMN = "URL";

    private static final String WHERE = "LANGUAGE=?";

    /**
     * Adapter constructor
     *
     * @param _context context
     */
    public IngredientAnalysisDataBaseAdapter(Context _context) {
        context = _context;
        dbHelper = new DataBaseHelper(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    /**
     * Method to open the Database
     *
     * @return adapter
     * @throws SQLException
     */
    public IngredientAnalysisDataBaseAdapter open() throws SQLException {
        db = dbHelper.getWritableDatabase();
        return this;
    }

    /**
     * Method to close the Database
     */
    public void close() {
        db.close();
    }

    /**
     * Method to insert a record in Table
     *
     * @param ingredientsAnalysisList source
     * @param language                selected locale
     * @return boolean
     */
    public boolean insertIngredientsAnalysis(List<IngredientsAnalysis.IngredientAnalysis> ingredientsAnalysisList, String language) {
        boolean result = true;
        long count = 0;
        try {
            ContentValues newValues = new ContentValues();
            for (IngredientsAnalysis.IngredientAnalysis ingredientAnalysis : ingredientsAnalysisList) {
                // Assign values for each column.
                newValues.put(LANGUAGE_COLUMN, language);
                newValues.put(CODE_COLUMN, ingredientAnalysis.id);
                newValues.put(NAME_COLUMN, ingredientAnalysis.name);
                newValues.put(PRODUCTS_COLUMN, ingredientAnalysis.products);
                newValues.put(KNOWN_COLUMN, ingredientAnalysis.known);
                newValues.put(URL_COLUMN, ingredientAnalysis.url);
                // Insert the row into your table
                db = dbHelper.getWritableDatabase();
                count = db.insert(TABLE_NAME, null, newValues);

            }
            Log.i(TAG, "Inserted ingredientAnalysis -> " + String.valueOf(count));
        } catch (Exception ex) {
            result = false;
            Log.e(TAG, "Error in IngredientAnalysisDataBaseAdapter.insertIngredientsAnalysis" + ex.getMessage());
        } finally {
            this.close();
        }
        return result;
    }

    /**
     * Method to get the contents
     *
     * @param language selected locale
     * @return ingredient analysis content
     */
    public List<IngredientsAnalysis.IngredientAnalysis> getIngredientsAnalysisByLanguage(String language) {
        List<IngredientsAnalysis.IngredientAnalysis> ingredientsAnalysisList = new ArrayList<>();
        Cursor cursor = null;
        try {
            db = dbHelper.getReadableDatabase();
            cursor = db.query(TABLE_NAME, null, WHERE, new String[]{language}, null, null, null);

            // Move to first row
            if (!cursor.moveToFirst())
                return ingredientsAnalysisList;
            do {
                IngredientsAnalysis.IngredientAnalysis ingredientAnalysis = new IngredientsAnalysis.IngredientAnalysis();
                ingredientAnalysis.id = cursor.getString(cursor.getColumnIndex(CODE_COLUMN));
                ingredientAnalysis.name = cursor.getString(cursor.getColumnIndex(NAME_COLUMN));
                ingredientAnalysis.products = cursor.getInt(cursor.getColumnIndex(PRODUCTS_COLUMN));
                ingredientAnalysis.known = cursor.getInt(cursor.getColumnIndex(KNOWN_COLUMN));
                ingredientAnalysis.url = cursor.getString(cursor.getColumnIndex(URL_COLUMN));
                ingredientsAnalysisList.add(ingredientAnalysis);
            } while (cursor.moveToNext());
        } finally {
            cursor.close();
            this.close();
        }
        return ingredientsAnalysisList;
    }
}
