package edu.uoc.tfg.nutriknow.sqlite.adapters;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.tfg.nutriknow.model.Labels;
import edu.uoc.tfg.nutriknow.sqlite.DataBaseHelper;

/**
 * Database adapter for labels
 */
public class LabelsDataBaseAdapter extends DataBaseAdapter {

    private static final String TAG = "LabelsDataBaseAdapter";

    static final int DATABASE_VERSION = 1;

    // SQL Statement to create a new database.
    public static final String TABLE_NAME = "LABEL";
    public static final String CREATE_TABLE = "CREATE TABLE IF NOT EXISTS LABEL (ID integer primary key autoincrement, LANGUAGE text, CODE text, NAME text, PRODUCTS integer, KNOWN integer, URL text); ";
    public static final String DROP_TABLE = "DROP TABLE IF EXISTS LABEL";

    // SQL Columns
    static final String LANGUAGE_COLUMN = "LANGUAGE";
    static final String CODE_COLUMN = "CODE";
    static final String NAME_COLUMN = "NAME";
    static final String PRODUCTS_COLUMN = "PRODUCTS";
    static final String KNOWN_COLUMN = "KNOWN";
    static final String URL_COLUMN = "URL";

    private static final String WHERE = "LANGUAGE=?";

    /**
     * Adapter constructor
     *
     * @param _context context
     */
    public LabelsDataBaseAdapter(Context _context) {
        context = _context;
        dbHelper = new DataBaseHelper(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    /**
     * Method to open the Database
     *
     * @return adapter
     * @throws SQLException
     */
    public LabelsDataBaseAdapter open() throws SQLException {
        db = dbHelper.getWritableDatabase();
        return this;
    }

    /**
     * Method to close the Database
     */
    public void close() {
        db.close();
    }

    /**
     * Method to insert a record in Table
     *
     * @param labelsList source
     * @param language   selected locale
     * @return boolean
     */
    public boolean insertLabels(List<Labels.Label> labelsList, String language) {
        boolean result = true;
        long count = 0;
        try {
            ContentValues newValues = new ContentValues();
            for (Labels.Label label : labelsList) {
                // Assign values for each column.
                newValues.put(LANGUAGE_COLUMN, language);
                newValues.put(CODE_COLUMN, label.id);
                newValues.put(NAME_COLUMN, label.name);
                newValues.put(PRODUCTS_COLUMN, label.products);
                newValues.put(KNOWN_COLUMN, label.known);
                newValues.put(URL_COLUMN, label.url);
                // Insert the row into your table
                db = dbHelper.getWritableDatabase();
                count = db.insert(TABLE_NAME, null, newValues);

            }
            Log.i(TAG, "Inserted labels -> " + String.valueOf(count));
        } catch (Exception ex) {
            result = false;
            Log.e(TAG, "Error in LabelsDataBaseAdapter.insertLabels" + ex.getMessage());
        } finally {
            this.close();
        }
        return result;
    }

    /**
     * Method to get the contents
     *
     * @param language selected locale
     * @return labels content
     */
    public List<Labels.Label> getLabelsByLanguage(String language) {
        List<Labels.Label> labelsList = new ArrayList<>();
        Cursor cursor = null;
        try {
            db = dbHelper.getReadableDatabase();
            cursor = db.query(TABLE_NAME, null, WHERE, new String[]{language}, null, null, null);

            // Move to first row
            if (!cursor.moveToFirst())
                return labelsList;
            do {
                Labels.Label label = new Labels.Label();
                label.id = cursor.getString(cursor.getColumnIndex(CODE_COLUMN));
                label.name = cursor.getString(cursor.getColumnIndex(NAME_COLUMN));
                label.products = cursor.getInt(cursor.getColumnIndex(PRODUCTS_COLUMN));
                label.known = cursor.getInt(cursor.getColumnIndex(KNOWN_COLUMN));
                label.url = cursor.getString(cursor.getColumnIndex(URL_COLUMN));
                labelsList.add(label);
            } while (cursor.moveToNext());
        } finally {
            cursor.close();
            this.close();
        }
        return labelsList;
    }
}
