package edu.uoc.tfg.nutriknow.sqlite.adapters;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import edu.uoc.tfg.nutriknow.model.WorkerData;
import edu.uoc.tfg.nutriknow.sqlite.DataBaseHelper;

/**
 * Database adapter for worker required data
 */
public class WorkerDataBaseAdapter extends DataBaseAdapter {

    private static final String TAG = "WorkerAdapter";

    static final int DATABASE_VERSION = 1;

    // SQL Statement to create a new database.
    public static final String TABLE_NAME = "WORKER_DATA";
    public static final String CREATE_TABLE = "CREATE TABLE IF NOT EXISTS WORKER_DATA (ID integer primary key autoincrement, USER_UUID text, FIREBASE_TOKEN text); ";
    public static final String DROP_TABLE = "DROP TABLE IF EXISTS WORKER_DATA";

    // SQL Columns
    static final String USER_UUID_COLUMN = "USER_UUID";
    static final String FIREBASE_TOKEN_COLUMN = "FIREBASE_TOKEN";

    /**
     * Adapter constructor
     *
     * @param _context context
     */
    public WorkerDataBaseAdapter(Context _context) {
        context = _context;
        dbHelper = new DataBaseHelper(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    /**
     * Method to open the Database
     *
     * @return adapter
     * @throws SQLException
     */
    public WorkerDataBaseAdapter open() throws SQLException {
        db = dbHelper.getWritableDatabase();
        return this;
    }

    /**
     * Method to close the Database
     */
    public void close() {
        db.close();
    }

    /**
     * Method to insert a record in Table
     *
     * @param userUuid      user uuid
     * @param firebaseToken firebase token
     * @return boolean
     */
    public boolean insertWorkerData(String userUuid, String firebaseToken) {

        boolean result = true;
        long count = 0;
        try {
            deleteWorkerData();

            ContentValues newValues = new ContentValues();
            newValues.put(USER_UUID_COLUMN, userUuid);
            newValues.put(FIREBASE_TOKEN_COLUMN, firebaseToken);

            db = dbHelper.getWritableDatabase();
            count = db.insert(TABLE_NAME, null, newValues);
            Log.i(TAG, "Inserted workerData -> " + String.valueOf(count));
        } catch (Exception ex) {
            result = false;
            Log.e(TAG, "Error in WorkerDataBaseAdapter.insertWorkerData" + ex.getMessage());
        } finally {
            this.close();
        }
        return result;
    }

    /**
     * Method to delete content of the table
     *
     * @return number of rows deleted
     */
    public int deleteWorkerData() {
        db = dbHelper.getWritableDatabase();
        int numberOFEntriesDeleted = db.delete(TABLE_NAME, null, null);
        Log.i(TAG, String.valueOf(numberOFEntriesDeleted));
        this.close();
        return numberOFEntriesDeleted;
    }

    /**
     * Method to get the contents
     *
     * @return worker data content
     */
    public WorkerData getWorkerData() {
        WorkerData workerData = null;
        Cursor cursor = null;
        try {
            db = dbHelper.getReadableDatabase();
            cursor = db.query(TABLE_NAME, null, null, null, null, null, null);

            // Move to first row
            if (!cursor.moveToFirst())
                return workerData;
            do {
                workerData = new WorkerData();
                workerData.setUserUuid(cursor.getString(cursor.getColumnIndex(USER_UUID_COLUMN)));
                workerData.setFirebaseToken(cursor.getString(cursor.getColumnIndex(FIREBASE_TOKEN_COLUMN)));
            } while (cursor.moveToNext());
        } finally {
            cursor.close();
            this.close();
        }
        return workerData;
    }
}
