package edu.uoc.tfg.nutriknow.tasks;

import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.content.Context;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.work.Worker;
import androidx.work.WorkerParameters;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.api.OpenFoodFactsClient;
import edu.uoc.tfg.nutriknow.api.interfaces.OpenFoodFactsService;
import edu.uoc.tfg.nutriknow.model.Additives;
import edu.uoc.tfg.nutriknow.model.Allergens;
import edu.uoc.tfg.nutriknow.model.CompleteProductData;
import edu.uoc.tfg.nutriknow.model.Favorite;
import edu.uoc.tfg.nutriknow.model.IngredientsAnalysis;
import edu.uoc.tfg.nutriknow.model.NutrientsLevel;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.model.Profile;
import edu.uoc.tfg.nutriknow.model.Search;
import edu.uoc.tfg.nutriknow.model.WorkerData;
import edu.uoc.tfg.nutriknow.sqlite.adapters.WorkerDataBaseAdapter;
import edu.uoc.tfg.nutriknow.utils.DatabaseCatalogUtils;
import edu.uoc.tfg.nutriknow.utils.ProductInfoUtils;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * Background service for looking for changes in favorite product class
 */
public class CheckForChangesBackgroundTask extends Worker {

    private static final String TAG = "CheckForChangesBgTask";

    private static final String ES = "es";
    private static final String PERIODICITY_D = "D";
    private static final String PERIODICITY_W = "W";
    private static final String PERIODICITY_N = "N";

    private static String token;
    private static String userUuid;
    private DatabaseReference dbReference;

    /**
     * Class constructor
     *
     * @param context
     * @param workerParams
     */
    public CheckForChangesBackgroundTask(@NonNull Context context, @NonNull WorkerParameters workerParams) {
        super(context, workerParams);
    }

    /**
     * Main method for starting the periodic task
     *
     * @return success result
     */
    @NonNull
    @Override
    public Result doWork() {
        // Log begin of the background task
        Log.i(TAG, "Background task begin");
        // Retrieve required data from local database
        WorkerDataBaseAdapter workerDataBaseAdapter = new WorkerDataBaseAdapter(getApplicationContext());
        WorkerData workerData = DatabaseCatalogUtils.retrieveWorkerDataLocalDatabase(workerDataBaseAdapter);
        if (workerData != null) {
            token = workerData.getFirebaseToken();
            userUuid = workerData.getUserUuid();

            if (token != null && userUuid != null) {
                // Get firebase database instance and retrieve root user position
                FirebaseDatabase database = FirebaseDatabase.getInstance();
                dbReference = database.getReference("/users");
                dbReference.child(userUuid).addListenerForSingleValueEvent(new ValueEventListener() {
                    @Override
                    public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                        // Retrieve searches
                        List<Search> searches = new ArrayList<>();
                        for (DataSnapshot productSnapshot : dataSnapshot.child("/searches").getChildren()) {
                            searches.add(productSnapshot.getValue(Search.class));
                        }
                        // Retrieve favorite products
                        List<Favorite> favorites = new ArrayList<>();
                        for (DataSnapshot productSnapshot : dataSnapshot.child("/favorites").getChildren()) {
                            favorites.add(productSnapshot.getValue(Favorite.class));
                        }
                        // For each of those favorite products
                        for (Favorite favorite : favorites) {
                            Search search = null;
                            for (Search srch : searches) {
                                if (srch.getProductCode().equals(favorite.getProductCode())) {
                                    search = srch;
                                    break;
                                }
                            }
                            // Check if there is some date configured (if not, notifications are not allowed by the user)
                            if (favorite.getNextDateToCheckProductInfo() != null) {
                                Date updateDate = favorite.getNextDateToCheckProductInfo();
                                Date todayDate = new Date();
                                Calendar calUpdate = Calendar.getInstance();
                                calUpdate.setTime(updateDate);
                                Calendar calToday = Calendar.getInstance();
                                calToday.setTime(todayDate);
                                // Check if the scheduled date corresponds with today
                                boolean sameDay = calUpdate.get(Calendar.DAY_OF_YEAR) == calToday.get(Calendar.DAY_OF_YEAR) &&
                                        calUpdate.get(Calendar.YEAR) == calToday.get(Calendar.YEAR);
                                if (sameDay) {
                                    // Get product corresponding with favorite register
                                    Product databaseProduct = null;
                                    for (DataSnapshot productSnapshot : dataSnapshot.child("/products").getChildren()) {
                                        Product prod = productSnapshot.getValue(Product.class);
                                        if (prod.getSummaryData().getCode().equals(favorite.getProductCode())) {
                                            databaseProduct = prod;
                                            break;
                                        }
                                    }
                                    if (databaseProduct != null) {
                                        // Get product information from api
                                        retrieveProductInformation(databaseProduct, favorite, search);
                                    }
                                }
                            }
                        }
                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError databaseError) {
                        Log.e(TAG, "Database error: " + databaseError.toException());
                    }
                });
            }
        }

        return Result.success();
    }

    /**
     * Method for sending push notifications
     *
     * @param product selected product
     */
    private void sendNotification(Product product) {
        // Notification manager creation
        NotificationManager notificationManager =
                (NotificationManager) getApplicationContext().getSystemService(Context.NOTIFICATION_SERVICE);
        // Since android Oreo notification channel is needed.
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            NotificationChannel channel = new NotificationChannel("channelId",
                    "ChannelName",
                    NotificationManager.IMPORTANCE_HIGH);
            channel.setDescription("WorkManger");
            notificationManager.createNotificationChannel(channel);
        }
        // Set product name depending on selected locale
        String productName = "";
        Locale currentLocale = getApplicationContext().getResources().getConfiguration().locale;
        if (ES.equals(currentLocale.getLanguage())) {
            if (product.getSummaryData().getProductName_es() != null && !product.getSummaryData().getProductName_es().isEmpty()) {
                productName = product.getSummaryData().getProductName_es();
            } else {
                if (product.getSummaryData().getProductName_en() != null && !product.getSummaryData().getProductName_en().isEmpty()) {
                    productName = product.getSummaryData().getProductName_en();
                } else {
                    productName = getApplicationContext().getString(R.string.empty_product_name);
                }
            }
        } else {
            if (product.getSummaryData().getProductName_en() != null && !product.getSummaryData().getProductName_en().isEmpty()) {
                productName = product.getSummaryData().getProductName_en();
            } else {
                if (product.getSummaryData().getProductName_es() != null && !product.getSummaryData().getProductName_es().isEmpty()) {
                    productName = product.getSummaryData().getProductName_es();
                } else {
                    productName = getApplicationContext().getString(R.string.empty_product_name);
                }
            }
        }
        String productCode = product.getSummaryData().getCode();
        // Call to asynchronous task for sending notifications
        new SendNotificationAsyncTask(getApplicationContext()).execute(token, productCode, productName);
    }

    /**
     * Retrieve product information and compare with stored info
     *
     * @param databaseProduct selected product
     * @param favorite        selected favorite
     * @param search          selected search
     */
    private void retrieveProductInformation(Product databaseProduct, Favorite favorite, Search search) {
        // Define endpoint to use
        OpenFoodFactsService apiService = OpenFoodFactsClient.getEsClient().create(OpenFoodFactsService.class);

        Call<CompleteProductData> call = apiService.getProductInformation(databaseProduct.getSummaryData().getCode());
        call.enqueue(new Callback<CompleteProductData>() {
            @Override
            public void onResponse(Call<CompleteProductData> call, Response<CompleteProductData> response) {
                // Retrieve api response
                CompleteProductData resource = response.body();

                // Comparison begins
                boolean comparisonResult = true;
                // Allergens comparison list sizes
                List<String> allergensRetrieved = resource.product.allergens_tags;
                List<Allergens.Allergen> allergensDatabase =
                        databaseProduct.getIngredientsData().getAllergens_es();
                if (allergensRetrieved.size() != allergensDatabase.size()) {
                    comparisonResult = false;
                }
                // Traces comparison list sizes
                List<String> tracesRetrieved = resource.product.traces_tags;
                List<Allergens.Allergen> tracesDatabase =
                        databaseProduct.getIngredientsData().getTraces_es();
                if (tracesRetrieved.size() != tracesDatabase.size()) {
                    comparisonResult = false;
                }
                // Completeness comparison
                float completenessRetrieved = resource.product.completeness;
                float completenessDatabase = databaseProduct.getSummaryData().getCompleteness();
                if (completenessRetrieved != completenessDatabase) {
                    comparisonResult = false;
                }
                // Ingredients Analysis comparison list sizes
                List<String> ingredientsAnalysisRetrieved = resource.product.ingredients_analysis_tags;
                List<IngredientsAnalysis.IngredientAnalysis> ingredientAnalysesDatabase =
                        databaseProduct.getSummaryData().getIngredientsAnalysis_en();
                if (ingredientsAnalysisRetrieved.size() != ingredientAnalysesDatabase.size()) {
                    comparisonResult = false;
                }
                // Number of additives comparison
                int additivesCountRetrieved = ProductInfoUtils.checkIfNullOrEmptyInteger(resource.product.additives_n);
                int additivesCountDatabase = databaseProduct.getIngredientsData().getAdditivesCount();
                if (additivesCountRetrieved != additivesCountDatabase) {
                    comparisonResult = false;
                }
                // Additives comparison list sizes
                List<String> additivesRetrieved = resource.product.additives_tags;
                List<Additives> additivesDatabase = databaseProduct.getIngredientsData().getAdditives();
                if (additivesRetrieved.size() != additivesDatabase.size()) {
                    comparisonResult = false;
                }
                // Nova group comparison
                int novaGroupRetrieved = ProductInfoUtils.checkIfNullOrEmptyInteger(resource.product.nova_group);
                int novaGroupDatabase = databaseProduct.getIngredientsData().getNovaGroup();
                if (novaGroupRetrieved != novaGroupDatabase) {
                    comparisonResult = false;
                }
                // Nutrients levels comparison list sizes
                List<String> nutrientLevelsRetrieved = resource.product.nutrient_levels_tags;
                List<NutrientsLevel.NutrientLevel> nutrientLevelsDatabase = databaseProduct.getNutritionalData().getNutrientLevels();
                if (nutrientLevelsRetrieved.size() != nutrientLevelsDatabase.size()) {
                    comparisonResult = false;
                }
                // Nutri grade comparison
                String nutriGradeRetrieved = resource.product.nutrition_grades;
                String nutriGradeDatabase = databaseProduct.getNutritionalData().getNutritionGrade();
                if (!nutriGradeRetrieved.equals(nutriGradeDatabase)) {
                    comparisonResult = false;
                }
                // If sizes are equals, content comparison begins
                if (comparisonResult) {
                    // Allergens comparison content
                    int allergensCount = 0;
                    for (Allergens.Allergen allergen : allergensDatabase) {
                        if (!allergensRetrieved.contains(allergen.id)) {
                            comparisonResult = false;
                            break;
                        } else {
                            allergensCount++;
                        }
                    }
                    if (allergensRetrieved.size() != allergensCount) {
                        comparisonResult = false;
                    }
                    // Traces comparison content
                    int tracesCount = 0;
                    for (Allergens.Allergen trace : tracesDatabase) {
                        if (!tracesRetrieved.contains(trace.id)) {
                            comparisonResult = false;
                            break;
                        } else {
                            tracesCount++;
                        }
                    }
                    if (tracesRetrieved.size() != tracesCount) {
                        comparisonResult = false;
                    }
                    // Ingredients analysis comparison content
                    int ingredientsAnalysisCount = 0;
                    for (IngredientsAnalysis.IngredientAnalysis ingredientsAnalysis : ingredientAnalysesDatabase) {
                        if (!ingredientsAnalysisRetrieved.contains(ingredientsAnalysis.id)) {
                            comparisonResult = false;
                            break;
                        } else {
                            ingredientsAnalysisCount++;
                        }
                    }
                    if (ingredientsAnalysisRetrieved.size() != ingredientsAnalysisCount) {
                        comparisonResult = false;
                    }
                    // Additives comparison content
                    int additivesCount = 0;
                    for (Additives additive : additivesDatabase) {
                        if (!additivesRetrieved.contains(additive.code)) {
                            comparisonResult = false;
                            break;
                        } else {
                            additivesCount++;
                        }
                    }
                    if (additivesRetrieved.size() != additivesCount) {
                        comparisonResult = false;
                    }
                    // Nutrients levels comparison content
                    int nutrientLevelsCount = 0;
                    for (NutrientsLevel.NutrientLevel nutrientLevel : nutrientLevelsDatabase) {
                        if (!nutrientLevelsRetrieved.contains(nutrientLevel.code)) {
                            comparisonResult = false;
                            break;
                        } else {
                            nutrientLevelsCount++;
                        }
                    }
                    if (nutrientLevelsRetrieved.size() != nutrientLevelsCount) {
                        comparisonResult = false;
                    }
                }
                boolean finalComparisonResult = comparisonResult;
                Log.i(TAG, databaseProduct.getSummaryData().getCode() + " has changes -> " + !finalComparisonResult);
                // Get user profile from database
                dbReference.child(userUuid).child("/profile").addListenerForSingleValueEvent(new ValueEventListener() {
                    @Override
                    public void onDataChange(DataSnapshot dataSnapshot) {
                        if (dataSnapshot.exists()) {
                            Profile profile = dataSnapshot.getValue(Profile.class);
                            // Set new look-for-changes date
                            Date date = new Date();
                            Calendar c = Calendar.getInstance();
                            String periodicity = profile.getPeriodicity();
                            switch (periodicity) {
                                case PERIODICITY_D:
                                    c.add(Calendar.DATE, 1);
                                    date = new Date(c.getTimeInMillis());
                                    break;
                                case PERIODICITY_W:
                                    c.add(Calendar.DATE, 7);
                                    date = new Date(c.getTimeInMillis());
                                    break;
                                case PERIODICITY_N:
                                    date = null;
                                    break;
                            }
                            favorite.setNextDateToCheckProductInfo(date);
                            favorite.setNew(!finalComparisonResult);
                            // Store new favorites date with new flag
                            dbReference.child(userUuid).child("/favorites")
                                    .child(favorite.getProductCode()).setValue(favorite,
                                    new DatabaseReference.CompletionListener() {
                                        @Override
                                        public void onComplete(DatabaseError databaseError,
                                                               DatabaseReference databaseReference) {
                                            if (databaseError == null) {

                                            } else {
                                                Log.e(TAG, "Database error: " + databaseError.toException());
                                            }
                                        }
                                    });

                            search.setNew(!finalComparisonResult);
                            // Store search with new flag
                            dbReference.child(userUuid).child("/searches")
                                    .child(search.getProductCode()).setValue(search,
                                    new DatabaseReference.CompletionListener() {
                                        @Override
                                        public void onComplete(DatabaseError databaseError,
                                                               DatabaseReference databaseReference) {
                                            if (databaseError == null) {

                                            } else {
                                                Log.e(TAG, "Database error: " + databaseError.toException());
                                            }
                                        }
                                    });
                            if (!finalComparisonResult) {
                                sendNotification(databaseProduct);
                            }
                        }
                    }

                    @Override
                    public void onCancelled(DatabaseError databaseError) {
                        Log.e(TAG, "Database error: " + databaseError.toException());
                    }
                });
            }

            @Override
            public void onFailure(Call<CompleteProductData> call, Throwable t) {
                call.cancel();
                Log.e(TAG, t.getMessage() + "");
            }
        });
    }
}
