package edu.uoc.tfg.nutriknow.utils;

import android.content.Context;

import com.google.android.material.textfield.TextInputLayout;

import java.util.List;
import java.util.regex.Pattern;

import edu.uoc.tfg.nutriknow.R;

/**
 * Utils class for input data validations
 */
public class InputValidationUtils {

    /**
     * Validate user name
     *
     * @param ctx       context
     * @param userInput input to validate
     * @param userName  text
     * @return boolean
     */
    public static boolean validateUserName(Context ctx, TextInputLayout userInput,
                                           String userName) {
        boolean valid = true;
        Pattern pattern = Pattern.compile("^[a-zA-Z ]+$");
        if (userName.isEmpty() || !pattern.matcher(userName).matches() || userName.length() < 6) {
            userInput.setError(ctx.getString(R.string.incorrect_username_error));
            userInput.setErrorEnabled(true);
            valid = false;
        } else {
            userInput.setError(null);
            userInput.setErrorEnabled(false);
        }
        return valid;
    }

    /**
     * Validate password
     *
     * @param ctx           context
     * @param passwordInput input to validate
     * @param password      text
     * @return boolean
     */
    public static boolean validatePassword(Context ctx, TextInputLayout passwordInput,
                                           String password) {
        boolean valid = true;
        if (password.isEmpty() || password.length() < 6) {
            passwordInput.setError(ctx.getString(R.string.incorrect_password_error));
            passwordInput.setErrorEnabled(true);
            valid = false;
        } else {
            passwordInput.setError(null);
            passwordInput.setErrorEnabled(false);
        }
        return valid;
    }

    /**
     * Validate email address
     *
     * @param ctx          context
     * @param emailInput   input to validate
     * @param emailAddress text
     * @return boolean
     */
    public static boolean validateEmailAddress(Context ctx, TextInputLayout emailInput,
                                               String emailAddress) {
        boolean valid = true;
        if (emailAddress.isEmpty() ||
                !android.util.Patterns.EMAIL_ADDRESS.matcher(emailAddress).matches()) {
            emailInput.setError(ctx.getString(R.string.incorrect_email_address_error));
            emailInput.setErrorEnabled(true);
            valid = false;
        } else {
            emailInput.setError(null);
            emailInput.setErrorEnabled(false);
        }
        return valid;
    }

    /**
     * Validate if the passwords are the same
     *
     * @param ctx                 context
     * @param passwordInput       first input to validate
     * @param repeatPasswordInput second input to validate
     * @param password            text of the first input
     * @param repeatPassword      text of the second input
     * @return boolean
     */
    public static boolean validateRepeatPassword(Context ctx, TextInputLayout passwordInput,
                                                 TextInputLayout repeatPasswordInput, String password,
                                                 String repeatPassword) {
        boolean valid = true;
        if (!password.equals(repeatPassword)) {
            passwordInput.setError(ctx.getString(R.string.incorrect_repeat_password_error));
            passwordInput.setErrorEnabled(true);
            repeatPasswordInput.setError(ctx.getString(R.string.incorrect_repeat_password_error));
            repeatPasswordInput.setErrorEnabled(true);
            valid = false;
        } else {
            passwordInput.setError(null);
            passwordInput.setErrorEnabled(false);
            repeatPasswordInput.setError(null);
            repeatPasswordInput.setErrorEnabled(false);
        }
        return valid;
    }

    /**
     * Validate allergen content
     *
     * @param ctx             context
     * @param allergenInput   input to validate
     * @param text            text to validate
     * @param allergensToShow source list
     * @return boolean
     */
    public static boolean validateAllergensContent(Context ctx, TextInputLayout allergenInput,
                                                   String text, List<String> allergensToShow) {
        boolean valid = true;
        if (text != null && !text.isEmpty()) {
            if (allergensToShow.contains(text)) {
                allergenInput.setError(null);
                allergenInput.setErrorEnabled(false);
            } else {
                allergenInput.setError(ctx.getString(R.string.incorrect_allergen_error));
                allergenInput.setErrorEnabled(true);
                valid = false;
            }
        }
        return valid;
    }

    /**
     * Validate selected allergens list
     *
     * @param ctx               context
     * @param allergenInput     input to validate
     * @param text              text to validate
     * @param selectedAllergens source list
     * @return boolean
     */
    public static boolean validateSelectedAllergensList(Context ctx, TextInputLayout allergenInput,
                                                        String text, List<String> selectedAllergens) {
        boolean valid = true;
        if (text != null && !text.isEmpty()) {
            if (!selectedAllergens.contains(text)) {
                allergenInput.setError(null);
                allergenInput.setErrorEnabled(false);
            } else {
                allergenInput.setError(ctx.getString(R.string.added_already_allergen_error));
                allergenInput.setErrorEnabled(true);
                valid = false;
            }
        }
        return valid;
    }

    /**
     * Validate selected allergen
     *
     * @param ctx           context
     * @param allergenInput input to validate
     * @param text          text to validate
     * @return boolean
     */
    public static boolean validateSelectedAllergen(Context ctx, TextInputLayout allergenInput,
                                                   String text) {
        boolean valid = true;
        if (text != null && !text.isEmpty()) {
            allergenInput.setError(null);
            allergenInput.setErrorEnabled(false);
        } else {
            allergenInput.setError(ctx.getString(R.string.incorrect_allergen_error));
            allergenInput.setErrorEnabled(true);
            valid = false;
        }
        return valid;
    }
}
