package edu.uoc.tfg.nutriknow.views;

import android.app.ProgressDialog;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.RelativeLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.work.Constraints;
import androidx.work.ExistingPeriodicWorkPolicy;
import androidx.work.NetworkType;
import androidx.work.PeriodicWorkRequest;
import androidx.work.WorkInfo;
import androidx.work.WorkManager;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.android.material.navigation.NavigationView;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.messaging.FirebaseMessaging;

import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import edu.uoc.tfg.nutriknow.NutriKnowApplication;
import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.api.OpenFoodFactsRepository;
import edu.uoc.tfg.nutriknow.model.DataHolder;
import edu.uoc.tfg.nutriknow.sqlite.adapters.WorkerDataBaseAdapter;
import edu.uoc.tfg.nutriknow.tasks.CheckForChangesBackgroundTask;
import edu.uoc.tfg.nutriknow.utils.DatabaseCatalogUtils;
import edu.uoc.tfg.nutriknow.views.activities.LoginActivity;
import edu.uoc.tfg.nutriknow.views.fragments.HomeFragment;
import edu.uoc.tfg.nutriknow.views.fragments.ProfileSetupFragment;

/**
 * Common activity class, implements load data interface and is extended for other activities
 */
public class CommonActivity extends AppCompatActivity implements LoadData {

    private static final String TAG = "CommonActivity";

    public FirebaseAuth firebaseAuth;
    public FirebaseAuth.AuthStateListener authStateListener;

    public FirebaseDatabase database;
    public DatabaseReference dbReference;

    public Toolbar toolbar;

    public RelativeLayout relativeLayout;

    public int view;
    public boolean postAuth;
    public boolean backAllowed = true;

    public DataHolder dataHolder;

    public DrawerLayout drawerLayout;
    public EndDrawerToggle drawerToggle;
    public NavigationView navigationView;
    public View headerView;

    public static Drawable defaultNavigationIcon;

    protected ExecutorService executorService = Executors.newFixedThreadPool(1);
    protected OpenFoodFactsRepository openFoodFactsRepository = new OpenFoodFactsRepository(this, executorService);

    private boolean finalized = false;
    private boolean failed = false;
    private static boolean executed = false;

    public Fragment callingFragment;

    public ProgressDialog progressDialog;

    public WorkerDataBaseAdapter workerDataBaseAdapter;
    public static boolean workerStarted = false;

    /**
     * Create view method
     *
     * @param savedInstanceState bundle
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);

        setContentView(view);

        relativeLayout = (RelativeLayout) findViewById(R.id.mainLayout);

        // Setup toolbar
        this.setupToolbar(backAllowed);
        // Get firebase auth instance and configure navigation when logout
        firebaseAuth = FirebaseAuth.getInstance();
        authStateListener = new FirebaseAuth.AuthStateListener() {
            @Override
            public void onAuthStateChanged(@NonNull FirebaseAuth firebaseAuth) {
                FirebaseUser user = firebaseAuth.getCurrentUser();
                if (postAuth && user == null) {
                    startActivity(new Intent(getApplicationContext(), LoginActivity.class));
                }
            }
        };
        // Get firebase database instance
        database = FirebaseDatabase.getInstance();
        dbReference = database.getReference("/users");
        // Get dataholder instance for storing user information
        dataHolder = DataHolder.getInstance();
        // Get worker adapter instance
        workerDataBaseAdapter = new WorkerDataBaseAdapter(getApplicationContext());
        // Get firebase cloud messaging token
        FirebaseMessaging.getInstance().getToken()
                .addOnCompleteListener(new OnCompleteListener<String>() {
                    @Override
                    public void onComplete(@NonNull Task<String> task) {
                        if (!task.isSuccessful()) {
                            Log.e(TAG, "Fetching FCM registration token failed", task.getException());
                            return;
                        }
                        // Get new FCM registration token
                        dataHolder.setFirebaseToken(task.getResult());
                    }
                });
    }

    /**
     * On pause
     */
    @Override
    protected void onPause() {
        super.onPause();
        if (isFinishing()) {
            overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
        }
        // Determine if app is in background
        NutriKnowApplication.activityPaused();
    }

    /**
     * On resume
     */
    @Override
    protected void onResume() {
        super.onResume();
        toolbar.getNavigationIcon().setTint(getColor(R.color.textColorPrimary));
        // Determine if app is in background
        NutriKnowApplication.activityResumed();
    }

    /**
     * Perform logout
     */
    public void logout() {
        firebaseAuth.signOut();
        dataHolder.setUser(null);
        dataHolder.setProfile(null);
        dataHolder.setProducts(null);
        dataHolder.setSearches(null);
        dataHolder.setFavorites(null);
        dataHolder.setCurrentLocale(null);
        Intent intent = new Intent(getApplicationContext(), LoginActivity.class);
        startActivity(intent);
        finish();
    }

    /**
     * Setup toolbar
     *
     * @param allowBack boolean
     */
    public void setupToolbar(boolean allowBack) {
        toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        if (defaultNavigationIcon == null) {
            defaultNavigationIcon = toolbar.getNavigationIcon();
        }
        toolbar.setNavigationIcon(defaultNavigationIcon);
        if (allowBack) {
            toolbar.setNavigationOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    toolbar.getNavigationIcon().setTint(getColor(R.color.textColorSecondary));
                    onBackPressed();
                }
            });
        }
    }

    /**
     * Load catalog data
     */
    protected void loadCatalogData() {
        // Start loading data from catalogs
        if (!executed) {
            openFoodFactsRepository.loadCatalogData();
            executed = true;
        }
    }

    /**
     * Check if catalog data is loaded
     *
     * @param progressDialog progress dialog
     */
    public void checkIfDataIsLoaded(ProgressDialog progressDialog) {
        // Get results
        Map<String, String[]> results = openFoodFactsRepository.getResults();
        if (results.size() == 0) {
            finalized = true;
        } else {
            int count = 0;
            // Check results values
            for (Map.Entry<String, String[]> entry : results.entrySet()) {
                if (OpenFoodFactsRepository.SUCCESS.equals(entry.getValue()[0])) {
                    count++;
                    Log.i(TAG, entry.getKey() + " -> " + entry.getValue()[0] + " - " + entry.getValue()[1] + " - " + entry.getValue()[2]);
                } else if (OpenFoodFactsRepository.RUNNING.equals(entry.getValue()[0])) {
                    Log.i(TAG, entry.getKey() + " -> " + entry.getValue()[0]);
                } else {
                    failed = true;
                    Log.i(TAG, entry.getKey() + " -> " + entry.getValue()[0]);
                }
            }
            // If all are in SUCCESS state
            if (count == results.size()) {
                finalized = true;
            }
        }
        // When finalized
        if (finalized) {
            // Retrieve data from database
            retrieveDataFromLocalDatabase();
            // Initialize worker for look for changes
            if (callingFragment instanceof HomeFragment && !workerStarted) {
                initializeWorker();
                workerStarted = true;
            }
            progressDialog.dismiss();
        } else {
            if (!failed) {
                // If not finalized, check again
                new Handler().postDelayed(() -> checkIfDataIsLoaded(progressDialog), 2000);
            } else {
                progressDialog.dismiss();
            }

        }

    }

    /**
     * Initialize worker
     */
    public void initializeWorker() {
        // Configure constraints
        Constraints constraints = new Constraints.Builder()
                .setRequiredNetworkType(NetworkType.UNMETERED)
                .build();
        // Save worker data in database
        DatabaseCatalogUtils.saveWorkerDataInLocalDatabase(workerDataBaseAdapter,
                dataHolder.getUser().getUuid(),
                dataHolder.getFirebaseToken());
        // Create perdiodic worker to be executed each 24 hours
        final PeriodicWorkRequest periodicWorkRequest =
                new PeriodicWorkRequest.Builder(CheckForChangesBackgroundTask.class,
                        1, TimeUnit.DAYS)
                        .setConstraints(constraints)
                        .setInitialDelay(15, TimeUnit.MINUTES)
                        .build();
        WorkManager workManager = WorkManager.getInstance(this);
        workManager.enqueueUniquePeriodicWork("checkForChangesJob",
                ExistingPeriodicWorkPolicy.KEEP, periodicWorkRequest);
        workManager.getWorkInfoByIdLiveData(periodicWorkRequest.getId())
                .observe(this, new Observer<WorkInfo>() {
                    @Override
                    public void onChanged(@Nullable WorkInfo workInfo) {
                        if (workInfo != null) {
                            Log.i(TAG, "Worker status changed to : " + workInfo.getState());

                        }
                    }
                });
    }

    /**
     * Retrieve data from local database depending on fragment
     */
    @Override
    public void retrieveDataFromLocalDatabase() {
        if (callingFragment instanceof ProfileSetupFragment) {
            ProfileSetupFragment fragment = (ProfileSetupFragment) callingFragment;
            fragment.retrieveDataFromLocalDatabase();
        }
    }
}
