package edu.uoc.tfg.nutriknow.views.activities;

import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import androidx.annotation.NonNull;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.material.textfield.TextInputLayout;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.ValueEventListener;
import com.joanzapata.iconify.IconDrawable;
import com.joanzapata.iconify.fonts.FontAwesomeIcons;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.model.Favorite;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.model.Profile;
import edu.uoc.tfg.nutriknow.model.Search;
import edu.uoc.tfg.nutriknow.model.User;
import edu.uoc.tfg.nutriknow.utils.IconDrawableUtils;
import edu.uoc.tfg.nutriknow.utils.InputValidationUtils;
import edu.uoc.tfg.nutriknow.utils.KeyboardUtils;
import edu.uoc.tfg.nutriknow.utils.ProgressDialogUtils;
import edu.uoc.tfg.nutriknow.utils.SnackBarUtils;
import edu.uoc.tfg.nutriknow.views.CommonActivity;

/**
 * Login activity class, extends from Common activity
 */
public class LoginActivity extends CommonActivity {

    private static final String TAG = "LoginActivity";

    private EditText emailText;
    private TextInputLayout emailInput;
    private EditText passwordText;
    private TextInputLayout passwordInput;
    private Button loginButton;
    private TextView signupLink;

    private boolean validEmail = false;
    private boolean validPassword = false;

    /**
     * Create view method
     *
     * @param savedInstanceState bundle
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        view = R.layout.login_activity;
        postAuth = false;
        backAllowed = false;
        super.onCreate(savedInstanceState);
        // Begin to load catalog if it's the first time using the app
        super.loadCatalogData();
        // Configure toolbar
        toolbar.setTitle(getString(R.string.login_activity_label));
        toolbar.setNavigationIcon(R.mipmap.toolbar_logo);
        // Reference views from layout
        emailInput = (TextInputLayout) findViewById(R.id.input_email);
        emailText = (EditText) findViewById(R.id.edit_email);
        // Add icon to input
        IconDrawable envelopeIcon = IconDrawableUtils
                .createIconDrawable(getApplicationContext(), FontAwesomeIcons.fa_envelope,
                        (int) emailText.getTextSize(), R.color.textColorPrimary, false);
        emailText.setCompoundDrawablesWithIntrinsicBounds(null, null, envelopeIcon, null);
        // Add content validation functionality
        emailText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                validEmail = InputValidationUtils
                        .validateEmailAddress(getApplicationContext(), emailInput, String.valueOf(s));
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (validate()) {
                    loginButton.setEnabled(true);
                } else {
                    loginButton.setEnabled(false);
                }
            }
        });

        passwordInput = (TextInputLayout) findViewById(R.id.input_password);
        passwordText = (EditText) findViewById(R.id.edit_password);
        // Add content validation functionality
        passwordText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                validPassword = InputValidationUtils
                        .validatePassword(getApplicationContext(), passwordInput, String.valueOf(s));
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (validate()) {
                    loginButton.setEnabled(true);
                } else {
                    loginButton.setEnabled(false);
                }
            }
        });
        // When enter is pressed in the keyboard, submit or pass to next field
        passwordText.setOnKeyListener(new View.OnKeyListener() {
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                if (validate()) {
                    if (event.getAction() == KeyEvent.ACTION_DOWN) {
                        switch (keyCode) {
                            case KeyEvent.KEYCODE_DPAD_CENTER:
                            case KeyEvent.KEYCODE_ENTER:
                                login();
                                return true;
                            default:
                                break;
                        }
                    }
                }
                return false;
            }
        });
        // Add login functionality
        loginButton = (Button) findViewById(R.id.btn_login);
        loginButton.setEnabled(false);
        loginButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                login();
            }
        });
        // Add register user functionality
        signupLink = (TextView) findViewById(R.id.link_signup);
        signupLink.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                signupLink.setTextColor(getColor(R.color.textColorSecondary));
                // Start the register activity
                Intent intent = new Intent(getApplicationContext(), RegisterActivity.class);
                startActivity(intent);
                overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
            }
        });
    }

    /**
     * Disable going back to the splash screen
     */
    @Override
    public void onBackPressed() {
        moveTaskToBack(true);
    }

    /**
     * In on resume, restore color for register link
     */
    @Override
    protected void onResume() {
        super.onResume();
        signupLink.setTextColor(getColor(R.color.textColorPrimary));
    }

    /**
     * Perform login
     */
    private void login() {
        // Validate form content
        if (!validate()) {
            onLoginFailed();
            return;
        }
        // Hide keyboard and clear inputs
        KeyboardUtils.hideKeyBoard(getApplicationContext(), LoginActivity.this);
        emailText.clearFocus();
        passwordText.clearFocus();

        loginButton.setEnabled(false);

        progressDialog =
                ProgressDialogUtils.createProgressDialog(this, R.string.login_authenticating_message);
        progressDialog.show();

        String email = emailText.getText().toString();
        String password = passwordText.getText().toString();
        // Sign in in firebase
        firebaseAuth.signInWithEmailAndPassword(email, password)
                .addOnSuccessListener(new OnSuccessListener<AuthResult>() {
                    @Override
                    public void onSuccess(@NonNull AuthResult result) {
                        // if login is successful, retrieve user information to dataHolder
                        dbReference.child(firebaseAuth.getCurrentUser().getUid()).addListenerForSingleValueEvent(new ValueEventListener() {
                            @Override
                            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                                if (firebaseAuth.getCurrentUser() != null) {
                                    // Retrieve user and profile from database
                                    dataHolder.setUser(dataSnapshot.child("/user").getValue(User.class));
                                    dataHolder.getUser().setUuid(firebaseAuth.getCurrentUser().getUid());
                                    dataHolder.setProfile(dataSnapshot.child("/profile").getValue(Profile.class));
                                    List<Product> products = new ArrayList<>();
                                    for (DataSnapshot productSnapshot : dataSnapshot.child("/products").getChildren()) {
                                        products.add(productSnapshot.getValue(Product.class));
                                    }
                                    dataHolder.setProducts(products);
                                    List<Search> searches = new ArrayList<>();
                                    for (DataSnapshot productSnapshot : dataSnapshot.child("/searches").getChildren()) {
                                        searches.add(productSnapshot.getValue(Search.class));
                                    }
                                    dataHolder.setSearches(searches);
                                    List<Favorite> favorites = new ArrayList<>();
                                    for (DataSnapshot productSnapshot : dataSnapshot.child("/favorites").getChildren()) {
                                        favorites.add(productSnapshot.getValue(Favorite.class));
                                    }
                                    dataHolder.setFavorites(favorites);
                                    onLoginSuccess();
                                    progressDialog.dismiss();
                                }
                            }

                            @Override
                            public void onCancelled(@NonNull DatabaseError databaseError) {
                                Log.e(TAG, "Database error: " + databaseError.toException());
                                onLoginFailed();
                                progressDialog.dismiss();
                            }
                        });
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception exception) {
                        // User cannot sign in and an exception is thrown
                        Log.e(TAG, exception.getMessage());
                        onLoginFailed();
                        progressDialog.dismiss();
                    }
                });
    }

    /**
     * If login is successful, redirect to welcome activity
     */
    private void onLoginSuccess() {
        loginButton.setEnabled(true);
        SnackBarUtils
                .showSnackBar(this, relativeLayout, getString(R.string.login_successful_message),
                        (int) emailText.getTextSize(), false);

        Intent intent = new Intent(getApplicationContext(), WelcomeActivity.class);
        startActivity(intent);
        overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
        finish();
    }

    /**
     * If login fails, show snackbar
     */
    private void onLoginFailed() {
        SnackBarUtils
                .showSnackBar(this, relativeLayout, getString(R.string.login_failed_error),
                        (int) emailText.getTextSize(), true);
        loginButton.setEnabled(false);
    }

    /**
     * Validate form values
     *
     * @return boolean
     */
    private boolean validate() {
        return validEmail && validPassword;
    }

}