package edu.uoc.tfg.nutriknow.views.fragments;

import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.viewpager.widget.ViewPager;

import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.android.material.tabs.TabLayout;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.adapters.TabsPagerAdapter;
import edu.uoc.tfg.nutriknow.model.Favorite;
import edu.uoc.tfg.nutriknow.model.Product;
import edu.uoc.tfg.nutriknow.utils.ProductInfoUtils;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

/**
 * Product info fragment class
 */
public class ProductInfoFragment extends Fragment {

    private static final String TITLE = "title";
    private static final String TAG = "ProductInfoFragment";
    private static final String ARG_PRODUCT = "product";

    private boolean isInFavs = false;

    private Fragment fragmentBefore;
    private ProductInfoFragment productInfoFragment;
    private RelativeLayout relativeLayout;
    public ViewPager viewPager;
    private WelcomeActivity welcomeActivity;
    public TabsPagerAdapter sectionsPagerAdapter;
    private TextView textView;

    private Product product;

    /**
     * Class constructor
     */
    public ProductInfoFragment() {
    }

    /**
     * Create a new instance
     *
     * @param title fragment title
     * @param prod product
     * @return fragment
     */
    public static ProductInfoFragment newInstance(String title, Product prod) {
        ProductInfoFragment fragment = new ProductInfoFragment();
        Bundle args = new Bundle();
        args.putString(TITLE, title);
        args.putSerializable(ARG_PRODUCT, prod);
        fragment.setArguments(args);
        return fragment;
    }

    /**
     * Create view
     *
     * @param savedInstanceState bundle
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            product = (Product) getArguments().getSerializable(ARG_PRODUCT);
        }
    }

    /**
     * Create view
     *
     * @param inflater           inflater
     * @param container          container
     * @param savedInstanceState bundle
     * @return view
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        final View view = inflater.inflate(R.layout.product_info_fragment, container, false);

        welcomeActivity = (WelcomeActivity) getActivity();
        welcomeActivity.prepareToolbarForFragments();

        productInfoFragment = (ProductInfoFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);

        fragmentBefore = welcomeActivity.callingFragment;
        welcomeActivity.callingFragment = productInfoFragment;

        // Reference to layout views and hide or display them depending on their values
        relativeLayout = (RelativeLayout) productInfoFragment.getActivity()
                .findViewById(R.id.mainLayout);

        textView = (TextView) view.findViewById(R.id.item);
        // Configure toolbar
        welcomeActivity.toolbar.setNavigationIcon(welcomeActivity.defaultNavigationIcon);
        welcomeActivity.toolbar.getNavigationIcon().setTint(getActivity().getColor(R.color.textColorPrimary));
        welcomeActivity.toolbar.setNavigationOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                goToPreviousScreen();
            }
        });

        // Configure tablayout and view pager
        sectionsPagerAdapter = new TabsPagerAdapter(getContext(), getChildFragmentManager(), product);

        viewPager = view.findViewById(R.id.view_pager);
        viewPager.setAdapter(sectionsPagerAdapter);
        int numberOfPages = 3;
        viewPager.setOffscreenPageLimit(numberOfPages);

        TabLayout tabs = view.findViewById(R.id.tabs);
        tabs.setupWithViewPager(viewPager);

        // Configure fab
        this.setupFloatingButton(view);

        return view;
    }

    /**
     * On resume
     */
    @Override
    public void onResume() {
        super.onResume();
        // Update view pager
        if (viewPager != null && viewPager.getAdapter() != null) {
            viewPager.getAdapter().notifyDataSetChanged();
        }
    }

    /**
     * Setup floating button
     *
     * @param view
     */
    private void setupFloatingButton(View view) {
        final FloatingActionButton floatingActionButton = (FloatingActionButton) view.findViewById(R.id.fab_button);

        // Evaluate state and assign icon
        isInFavs = evaluateFloatingButtonState();
        if (isInFavs) {
            floatingActionButton.setImageDrawable(getResources().getDrawable(R.drawable.ic_menu_my_favorites));
        } else {
            floatingActionButton.setImageDrawable(getResources().getDrawable(R.drawable.ic_menu_my_favorites_full));
        }
        // Add fab functionality
        floatingActionButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                isInFavs = evaluateFloatingButtonState();

                if (isInFavs) {
                    floatingActionButton.setImageDrawable(getResources().getDrawable(R.drawable.ic_menu_my_favorites_full));

                    relativeLayout = (RelativeLayout) productInfoFragment.getActivity()
                            .findViewById(R.id.mainLayout);

                    ProductInfoUtils.deleteProductFromFavorites(product.getSummaryData().getCode(), getContext(), welcomeActivity,
                            relativeLayout, textView, null, null);

                } else {
                    floatingActionButton.setImageDrawable(getResources().getDrawable(R.drawable.ic_menu_my_favorites));

                    relativeLayout = (RelativeLayout) productInfoFragment.getActivity()
                            .findViewById(R.id.mainLayout);

                    ProductInfoUtils.addProductToFavorites(product, getContext(), welcomeActivity,
                            relativeLayout, textView, null);
                }
            }
        });
    }

    /**
     * Evaluate fab state
     *
     * @return boolean
     */
    private boolean evaluateFloatingButtonState() {
        boolean found = false;
        for (Favorite favorite : welcomeActivity.dataHolder.getFavorites()) {
            if (favorite.getProductCode().equals(product.getSummaryData().getCode())) {
                found = true;
                break;
            }
        }
        isInFavs = found;
        return isInFavs;
    }

    /**
     * Depending on the calling fragment, redirect to previous screen
     */
    private void goToPreviousScreen() {
        welcomeActivity.toolbar.getNavigationIcon().setTint(getActivity().getColor(R.color.textColorSecondary));
        Fragment fragment = null;
        if (fragmentBefore instanceof ScanFragment) {
            fragment = ScanFragment.newInstance(getString(R.string.scan_activity_label));
            getActivity().setTitle(getString(R.string.scan_activity_label));
        } else if (fragmentBefore instanceof SearchesFragment) {
            fragment = SearchesFragment.newInstance(getString(R.string.searches_activity_label));
            getActivity().setTitle(getString(R.string.searches_activity_label));
        } else {
            fragment = FavoritesFragment.newInstance(getString(R.string.favorites_activity_label));
            getActivity().setTitle(getString(R.string.favorites_activity_label));
        }
        FragmentManager fragmentManager = getFragmentManager();
        fragmentManager.beginTransaction().replace(R.id.fragment_layout, fragment).commit();

    }
}