package edu.uoc.tfg.nutriknow.views.fragments;

import android.Manifest;
import android.content.pm.PackageManager;
import android.hardware.Camera;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.os.Bundle;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;

import com.google.android.gms.vision.Detector;
import com.google.android.gms.vision.barcode.Barcode;
import com.google.android.gms.vision.barcode.BarcodeDetector;

import java.io.IOException;

import edu.uoc.tfg.nutriknow.R;
import edu.uoc.tfg.nutriknow.utils.ProductInfoUtils;
import edu.uoc.tfg.nutriknow.utils.SnackBarUtils;
import edu.uoc.tfg.nutriknow.views.CameraSource;
import edu.uoc.tfg.nutriknow.views.LoadData;
import edu.uoc.tfg.nutriknow.views.activities.WelcomeActivity;

/**
 * Scan fragment class, implements permissions request and load data interfaces
 */
public class ScanFragment extends Fragment implements ActivityCompat.OnRequestPermissionsResultCallback,
        LoadData {

    private static final String TITLE = "title";
    private static final int REQUEST_CAMERA_PERMISSION = 201;

    private WelcomeActivity welcomeActivity;
    private ScanFragment scanFragment;
    private RelativeLayout relativeLayout;

    private SurfaceView surfaceView;
    private BarcodeDetector barcodeDetector;
    private CameraSource cameraSource;
    private ImageView flashButton;
    private ImageView volumeButton;

    private ToneGenerator toneGen1;
    private TextView barcodeText;
    private Animation anim;
    private String barcodeData;
    private boolean flashLightStatus;
    private boolean volumeStatus;

    /**
     * Class constructor
     */
    public ScanFragment() {
    }

    /**
     * Create a new instance
     *
     * @param title fragment title
     * @return fragment
     */
    public static ScanFragment newInstance(String title) {
        ScanFragment fragment = new ScanFragment();
        Bundle args = new Bundle();
        args.putString(TITLE, title);
        fragment.setArguments(args);
        return fragment;
    }

    /**
     * Create view
     *
     * @param savedInstanceState bundle
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        final View view = inflater.inflate(R.layout.scan_fragment, container, false);

        welcomeActivity = (WelcomeActivity) getActivity();
        // Configure toolbar
        welcomeActivity.prepareToolbarForFragments();
        // Select options in bottombar and drawer
        welcomeActivity.navigationView.setCheckedItem(R.id.nav_item0);
        welcomeActivity.bottomNavigationView.getMenu().getItem(1).setChecked(true);

        FragmentManager fragmentManager = getFragmentManager();
        scanFragment = (ScanFragment) getFragmentManager()
                .findFragmentById(R.id.fragment_layout);

        welcomeActivity.callingFragment = scanFragment;
        // Reference layout views
        relativeLayout = (RelativeLayout) scanFragment.getActivity()
                .findViewById(R.id.mainLayout);

        toneGen1 = new ToneGenerator(AudioManager.STREAM_MUSIC, 100);

        barcodeText = view.findViewById(R.id.barcode_text);
        barcodeText.setVisibility(View.VISIBLE);
        // Add animation to text
        anim = new AlphaAnimation(0.0f, 1.0f);
        anim.setDuration(1000);
        anim.setStartOffset(20);
        anim.setRepeatMode(Animation.REVERSE);
        anim.setRepeatCount(Animation.INFINITE);
        barcodeText.startAnimation(anim);

        surfaceView = view.findViewById(R.id.surface_view);
        // Add icon and functionality to flash button
        flashButton = (ImageView) view.findViewById(R.id.flash_icon);
        flashButton.setImageDrawable(getActivity().getDrawable(R.drawable.ic_flash_on));
        flashButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                boolean deviceHasCameraFlash = getActivity().getPackageManager()
                        .hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH);
                if (deviceHasCameraFlash) {
                    if (flashLightStatus) {
                        cameraSource.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
                        flashLightStatus = false;
                        flashButton.setImageDrawable(getActivity().getDrawable(R.drawable.ic_flash_on));
                    } else {
                        cameraSource.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH);
                        flashLightStatus = true;
                        flashButton.setImageDrawable(getActivity().getDrawable(R.drawable.ic_flash_off));
                    }
                }
            }
        });
        // Add icon and functionality to volume button
        volumeButton = (ImageView) view.findViewById(R.id.volume_icon);
        volumeButton.setImageDrawable(getActivity().getDrawable(R.drawable.ic_volume_on));
        volumeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (volumeStatus) {
                    volumeStatus = false;
                    volumeButton.setImageDrawable(getActivity().getDrawable(R.drawable.ic_volume_on));
                } else {
                    volumeStatus = true;
                    volumeButton.setImageDrawable(getActivity().getDrawable(R.drawable.ic_volume_off));
                }
            }
        });
        // Initalize camera an barcode detector
        initializeDetectorsAndSources();

        return view;
    }

    /**
     * on view created
     *
     * @param view               view
     * @param savedInstanceState bundle
     */
    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        final WelcomeActivity welcomeActivity = (WelcomeActivity) getActivity();
        this.welcomeActivity = welcomeActivity;
        // Retrieve data from database
        this.retrieveDataFromLocalDatabase();
    }

    /**
     * Initalize camera an barcode detector
     */
    private void initializeDetectorsAndSources() {

        flashLightStatus = false;
        volumeStatus = false;
        // Asign drawables to icons
        flashButton.setImageDrawable(getActivity().getDrawable(R.drawable.ic_flash_on));
        volumeButton.setImageDrawable(getActivity().getDrawable(R.drawable.ic_volume_on));
        // Build detector
        barcodeDetector = new BarcodeDetector.Builder(getContext())
                .setBarcodeFormats(Barcode.ALL_FORMATS)
                .build();
        // Configure camera source
        cameraSource = new CameraSource.Builder(getContext(), barcodeDetector)
                .setRequestedPreviewSize(1920, 1080)
                .setFlashMode(Camera.Parameters.FLASH_MODE_OFF)
                .setFocusMode(Camera.Parameters.FOCUS_MODE_CONTINUOUS_PICTURE)
                .build();
        // add calback to surface view to stop camera when destroyed
        surfaceView.getHolder().addCallback(new SurfaceHolder.Callback() {
            @Override
            public void surfaceCreated(SurfaceHolder holder) {
                startCamera();
            }

            @Override
            public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
            }

            @Override
            public void surfaceDestroyed(SurfaceHolder holder) {
                cameraSource.stop();
            }
        });
        // Configure detector
        barcodeDetector.setProcessor(new Detector.Processor<Barcode>() {
            @Override
            public void release() {
            }

            @Override
            public void receiveDetections(Detector.Detections<Barcode> detections) {
                final SparseArray<Barcode> barcodes = detections.getDetectedItems();
                if (barcodes.size() != 0) {
                    if (barcodes.valueAt(0).email != null) {
                        barcodeText.removeCallbacks(null);
                        barcodeData = barcodes.valueAt(0).email.address;
                    } else {
                        barcodeData = barcodes.valueAt(0).displayValue;
                    }
                    if (volumeStatus) {
                        toneGen1.startTone(ToneGenerator.TONE_CDMA_PIP, 150);
                    }
                    // When a barcode is detected
                    if (barcodeData != null) {
                        welcomeActivity.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                barcodeDetector.release();
                                cameraSource.stop();
                                // Retrieve product information
                                welcomeActivity.retrieveProductInformation(barcodeData, null);
                            }
                        });
                    }
                }
            }
        });
    }

    /**
     * Request permission to use the device camera
     *
     * @param requestCode  code
     * @param permissions  permissions to request
     * @param grantResults results
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
                                           @NonNull int[] grantResults) {
        switch (requestCode) {
            // if requested
            case REQUEST_CAMERA_PERMISSION:
                if (grantResults.length > 0 &&
                        grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    startCamera();
                } else {
                    scanFragment = (ScanFragment) getFragmentManager()
                            .findFragmentById(R.id.fragment_layout);
                    relativeLayout = (RelativeLayout) scanFragment.getActivity()
                            .findViewById(R.id.mainLayout);

                    SnackBarUtils
                            .showSnackBar(getContext(), relativeLayout,
                                    getString(R.string.camera_permission_error),
                                    (int) barcodeText.getTextSize(), true);
                    anim.cancel();
                    barcodeText.setVisibility(View.GONE);
                }
                return;
        }
    }

    /**
     * Start camera
     */
    private void startCamera() {
        try {
            // if permission was granted, start camera
            if (ActivityCompat.checkSelfPermission(getContext(), Manifest.permission.CAMERA) == PackageManager.PERMISSION_GRANTED) {
                cameraSource.start(surfaceView.getHolder());
            } else {
                // if permission was not granted, ask for it again
                requestPermissions(new String[]{Manifest.permission.CAMERA},
                        REQUEST_CAMERA_PERMISSION);
            }
        } catch (IOException e) {
            SnackBarUtils
                    .showSnackBar(getContext(), relativeLayout,
                            getString(R.string.camera_permission_error),
                            (int) barcodeText.getTextSize(), true);
        }
    }

    /**
     * Retrieve data from database
     */
    @Override
    public void retrieveDataFromLocalDatabase() {
        ProductInfoUtils.retrieveDataFromLocalDatabase(welcomeActivity);
    }
}